/*
 ***********************************************************************
 * Copyright 2005-2006 by Freescale Semiconductor, Inc.
 * All modifications are confidential and proprietary information
 * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
 ***********************************************************************
 */

#ifndef FSR_AVI_DEMUX_H
#define FSR_AVI_DEMUX_H
/*==================================================================================================
     Header Name: AVI_Demux.h
     General Description: API functions definitions. 
====================================================================================================

====================================================================================================
                                         INCLUDE FILES
==================================================================================================*/

#include "smartdsp.h"

/*==================================================================================================
                                           CONSTANTS
==================================================================================================*/

/*==================================================================================================
                                            MACROS
==================================================================================================*/

#define FSR_AVI_DEMUX_VIDEO_STREAM                  0
#define FSR_AVI_DEMUX_AUDIO_STREAM                  1

/*==================================================================================================
                                             ENUMS
==================================================================================================*/

typedef enum
{
    FSR_AVI_DEMUX_NOVideo = 0,
    FSR_AVI_DEMUX_MPEG4_Video
} FSR_AVI_DEMUX_VIDEO_TYPE_T;

typedef enum
{
    FSR_AVI_DEMUX_NOAudio = 0,
    FSR_AVI_DEMUX_MP3Audio
} FSR_AVI_DEMUX_AUDIO_TYPE_T;

typedef enum {

	CHUNK_TYPE_UNKNOWN,
	CHUNK_TYPE_AUDIO,
	CHUNK_TYPE_UNCOMPRESSED,
	CHUNK_TYPE_VIDEO,
	CHUNK_TYPE_DRM,
	CHUNK_TEXT_TYPE_CAPTION,
	CHUNK_BITMAP_TYPE_CAPTION,
	CHUNK_TYPE_CHAPTER,
	CHUNK_TYPE_STANDARD_INDEX,
	CHUNK_TYPE_METADATA,
	CHUNK_TYPE_NONE,

} chunk_type_t;
/*==================================================================================================
                                 STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/

typedef struct _AVI_header_strh_t 
{
    unsigned int    fcc;
    int             cb;           // size of this structure - 8
    unsigned int    fccType;      // stream type codes
    unsigned int    fccHandler;
    int             dwFlags;
    short int       wPriority;
    short int       wLanguage;
    int             dwInitialFrames;
    int             dwScale;
    int             dwRate;       // dwRate/dwScale is stream tick rate in ticks/sec
    int             dwStart;
    int             dwLength;
    int             dwSuggestedBufferSize;
    int             dwQuality;
    int             dwSampleSize;
    struct _sthrcFrame
    {
        short int   left;
        short int   top;
        short int   right;
        short int   bottom;
    }   rcFrame;
} AVI_header_strh_t;

typedef struct aviin_stream_entry_s 
{
	AVI_header_strh_t strh;
	chunk_type_t type;
} aviin_stream_entry_t;

typedef struct FSR_AVI_DEMUX_SPEC_S
{
    UWord32                     noStreams;
    FSR_AVI_DEMUX_VIDEO_TYPE_T  video_dec;
    UWord32                     video_width;
    UWord32                     video_height;
    UWord32                     video_freq;
    float                       video_framerate;
    UWord32                     video_compression;
    

    FSR_AVI_DEMUX_AUDIO_TYPE_T  audio_dec;
    UWord32                     audio_freq;
    UWord16                     audio_channels;
    UWord32                     audio_bitrate;
    UWord32                     audio_rate;
    UWord16                     format_tag;
    aviin_stream_entry_t        stream_index[2];/* Hard Coded. */
   

} FSR_AVI_DEMUX_SPEC_T;


typedef struct FSR_AVI_DEMUX_CTRL_S
{
    STD_BD_T            MemPerInst;                 /*channel space (persistent memory)*/
    const STD_BD_T      * paMemSelect;              /*memory partitions (scratch, shared memories etc)*/
    UWord16             usiCntMemSelect;            /*number of memory partitions*/ 
    
    /*the rewind information in the control structure can be used to command a different rewind point*/
    
    UWord32             uliRewFillIn;               /*rewind information for input buffer  (Offset in BD)*/
    UWord32             uliRewFillOut;              /*rewind information for output buffer (Offset in BD)*/
    UWord16             usiRewBDIn;                 /*rewind information for input buffer  (Bd index)*/
    UWord16             usiRewBDOut;                /*rewind information for output buffer (Bd index)*/

    
    UWord16             usiCntIn;                   /*number of sets in input space*/
    UWord16             usiCntOut;                  /*number of sets in output space*/
    UWord16             usiInRetMode;               /* Input return mode */
    UWord16             usiOutRetMode;              /* Output return mode */

}FSR_AVI_DEMUX_CTRL_T;

typedef struct MFW_METADATA_INFO_S
{
 /* metadata information */
    char* title;
    char* copyright;
    char* comment;
    char* artist;
    char* software;
    char* product;
    char* language;
    char* subject;
    char* source;
    char* creation;
    char* genre;
}MFW_METADATA_INFO_T;

typedef struct FSR_AVI_DEMUX_STAT_S
{    
    /*rewind information: stores the beginning of the current frame to be processed*/   
    UWord32                uliRewFillIn;               /*rewind information for input buffer  (Offset in BD)*/    
    UWord32                uliRewFillOut;              /*rewind information for output buffer (Offset in BD)*/
    UWord16                usiRewBDIn;                 /*rewind information for input buffer  (Bd index)*/
    UWord16                usiRewBDOut;                /*rewind information for output buffer (Bd index)*/
    
    /*end information: stores the ending of the current processed data (input and output). Can be used to signal
    the BD (index and fill) that generated a 'need more input' request or a 'need more output space' request*/
    //UWord32              uliEndFillIn;                /*OPT: end information for input buffer  (Offset in BD)*/
    //UWord32              uliEndFillOut;               /*OPT: end information for output buffer (Offset in BD)*/
    //UWord16              usiEndBDIn;                  /*OPT: end information for input buffer  (Bd index)*/
    //UWord16              usiEndBDOut;                 /*OPT: end information for output buffer (Bd index)*/       
    
    UWord16                usiEndBitstream;
    FSR_AVI_DEMUX_SPEC_T   specData;                    /*AVI streams information*/
    
    //UWord32              uliMemPerInst;               /*OPT: How many bytes of persistent memory are required */
    //UWord32 *            puliMemSelect;               /*OPT: How many bytes of additional memory are required */
    STD_VERSION_INFO_T     stLibVersion;
    MFW_METADATA_INFO_T    metadata;
}FSR_AVI_DEMUX_STAT_T;

/*==================================================================================================
                                 GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/

/*==================================================================================================
                                     FUNCTION PROTOTYPES
==================================================================================================*/

extern STD_RETURN_T FSR_AVI_DEMUX_Init(
        const FSR_AVI_DEMUX_CTRL_T * pstCtrl
);

extern STD_RETURN_T FSR_AVI_DEMUX_Close(
        const FSR_AVI_DEMUX_CTRL_T * pstCtrl,
        FSR_AVI_DEMUX_STAT_T * pstStat
);

extern STD_RETURN_T FSR_AVI_DEMUX_ProcessHDR(  
       const STD_BD_SET_T         * pstIn,
       STD_BD_SET_T               * pstOut,
       const FSR_AVI_DEMUX_CTRL_T * pstCtrl,
       FSR_AVI_DEMUX_STAT_T       * pstStat         
);

extern STD_RETURN_T FSR_AVI_DEMUX_Process(
       const STD_BD_SET_T           * pstIn,
       STD_BD_SET_T                 * pstOut,
       const FSR_AVI_DEMUX_CTRL_T   * pstCtrl,
       FSR_AVI_DEMUX_STAT_T         * pstStat
);

/*================================================================================================*/
#endif  /* FSR_AVI_DEMUX_H */
