
 /************************************************************************
  * Copyright 2005-2006 by Freescale Semiconductor, Inc.
  * All modifications are confidential and proprietary information
  * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
  ************************************************************************/

/*******************************************************************************
 *
 * (C) 2004 MOTOROLA INDIA ELECTRONICS LTD.
 *
 *   CHANGE HISTORY
 *   dd/mm/yy   Code Ver     Description                        Author
 *   --------   -------      -----------                        ------
 *   08/07/2004 01           Created                            Vishala
 *   05/08/2004 02           Counting bits used                 S.Nachiappan
 *                           from the bitstream
 *                           (enabled by defining BOOK_KEEP)
 *   19/08/2004 03           Added AACPD_Block_Params,           S.Nachiappan
 *                           which the decoder will receive
 *                           along with the block to be decoded.
 *                           AACPD_Block_Params contains
 *                           parameters similar to ADIF/ADTS
 *                           Header data.
 *   23/08/2004 04           Moved BitCreditType to             S.Nachiappan
 *                           aacplus_dec_interface.h
 *   09/09/2004 04           Added comments on                  S.Nachiappan
 *                           AACPD_Block_Params.
 *   02/02/2007 06          Level 4 Warnings removal            rks05c
 *                           Defect ID: TLSbo89610
 *   18/04/2007 07          Added ptr to FFT twiddle            Satish K Singh
 *                             factor table
 ******************************************************************************/
/**************************** Change History ***********************************
 *
 *   DD/MM/YYYY Code Ver     Description                        Author
 *   --------   -------      -----------                        ------
 *   08/06/2005    05        include header Sbrdecoder.h        Webber Wang
 *                           add sbr variables in global_str
 *   09/06/2005    05        add SBRD_global_struct,            Webber Wang
 *                           aacd_and add a pointer in struct
 *                           AACD_global_struct to pointer
 *                           to the said structure. remove
 *                           sbr variables
 *                           from AACD_global_struct
 *
 ******************************************************************************/

#ifndef _AAC_COMMON_H
#define _AAC_COMMON_H

#include "aac_fix_defs.h"
#include "aac_fix_types.h"
#include "fix_cd.h"
#include "aacd_tables.h"

#define NUM_BLOCK_LENS 2

#define USE_SCRATCH_MEMORY /*For reducing static memory*/

typedef struct
{
    Lfract         *pAACD_pow_4by3_table_lf;                       /* offset 0  */
    unsigned char  *tbl_ptr_AACD_n_lzeros_8bit;                    /* offset 1  */
    short          **tbl_ptr_AACD_bitrev_indices;                  /* offset 2  */
    short          *tbl_ptr_AACD_num_bitrev_sets;                  /* offset 3  */
    Lfract         *tbl_ptr_AACD_fft_radix4_twidfac;               /* offset 4  */
    Lfract         *tbl_ptr_AACD_prepost_twidfac;                  /* offset 5  */
    Lfract         *tbl_ptr_AACD_scale_fac_table_lf;               /* offset 6  */
    Lfract         *tbl_ptr_AACD_k_tab_lf;                         /* offset 7  */

    int            AACD_g_index;                                   /* offset 8  */
    Lfract         *tbl_ptr_AACD_KRAM_pow_4by3_table_lf;           /* offset 9  */
    Lfract         *tbl_ptr_AACD_tns_filt_state;                   /* offset 10 */
    BitstreamParam AACD_bs_param;                                  /* offset 11 */

    int            *tbl_ptr_AACD_pred_max_bands_tbl;               /* offset */
    int            (*tbl_ptr_AACD_tns_max_bands_tbl)[4];           /* offset */
    unsigned short *tbl_ptr_AACD_neg_mask;                         /* offset */
    short          *tbl_ptr_AACD_sgn_mask;                         /* offset */
    unsigned int   *tbl_ptr_AACD_hufftab_off;                      /* offset */
    Lfract         *((*tbl_ptr_AACD_windowPtr)[2]);                  /* offset */
    SR_Info        *tbl_ptr_AACD_samp_rate_info;                   /* offset */
    unsigned char  **tbl_ptr_AACD_huffstart;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable1;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable2;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable3;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable4;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable5;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable6;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable7;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable8;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable9;                       /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTable10;                      /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTableEsc;                     /* offset */
    unsigned char  *tbl_ptr_AACD_HuffTableScl;                     /* offset */

    short          *tbl_ptr_AACD_sfb_96_1024;                      /* offset */
    short          *tbl_ptr_AACD_sfb_96_128;                       /* offset */
    short          *tbl_ptr_AACD_sfb_64_1024;                      /* offset */
    short          *tbl_ptr_AACD_sfb_64_128;                       /* offset */
    short          *tbl_ptr_AACD_sfb_48_1024;                      /* offset */
    short          *tbl_ptr_AACD_sfb_48_128;                       /* offset */
    short          *tbl_ptr_AACD_sfb_32_1024;                      /* offset */
    short          *tbl_ptr_AACD_sfb_24_1024;                      /* offset */
    short          *tbl_ptr_AACD_sfb_24_128;                       /* offset */
    short          *tbl_ptr_AACD_sfb_16_1024;                      /* offset */
    short          *tbl_ptr_AACD_sfb_16_128;                       /* offset */
    short          *tbl_ptr_AACD_sfb_8_1024;                       /* offset */
    short          *tbl_ptr_AACD_sfb_8_128;                        /* offset */

    short          *tbl_ptr_AACD_bitrev_indices_256;               /* offset */
    short          *tbl_ptr_AACD_bitrev_indices_2048;              /* offset */

    Lfract         *tbl_ptr_AACD_window_fhg_long;
    Lfract         *tbl_ptr_AACD_window_fhg_short;
    Lfract         *tbl_ptr_AACD_window_dol_long;
    Lfract         *tbl_ptr_AACD_window_dol_short;

    Lfract         (*tbl_ptr_log2_coef)[4];
    Lfract         (*tbl_ptr_pow2_coef)[4];
    Lfract         *tbl_ptr_pow2_neg_coef;

    Lfract         *tbl_ptr_Coeff_CR_512;
    Lfract         *tbl_ptr_Coeff_CI_512;
    Lfract         *tbl_ptr_Coeff_CR_64;
    Lfract         *tbl_ptr_Coeff_CI_64;
    Lfract         *tbl_ptr_Coeff_PostTwid_CR_long;
    Lfract         *tbl_ptr_Coeff_PostTwid_CI_long;
    Lfract         *tbl_ptr_Coeff_PostTwid_CR_short;
    Lfract         *tbl_ptr_Coeff_PostTwid_CI_short;
#ifndef USE_SCRATCH_MEMORY
    Lfract         AACD_real[1024];
#else
    /*To be allocated by application - Pradeep*/
    Lfract         *AACD_real;
    Lfract           AACD_real1[220];

#endif
    Lfract         AACD_tns_filt_state[16];
#if defined(MAIN) || defined (SSR)
    Float        AACD_buf_coef[2][LN2];
    Lfract       AACD_buf_data[Chans][LN2];
#endif
#ifndef TARGET
    Lfract       AACD_buf_state[Chans][LN2];
#ifndef USE_SCRATCH_MEMORY
    Lfract       AACD_buf_coef_lf[Chans][LN2];
#else
    Lfract       *AACD_buf_coef_lf[Chans];
#endif /*USE_SCRATCH_MEMORY*/
#else
    Lfract       AACD_buf_state[2][LN2/2];
#ifndef USE_SCRATCH_MEMORY
    Lfract       AACD_buf_coef_lf[2][LN2];
#else
    Lfract      *AACD_buf_coef_lf[2];
#endif /*USE_SCRATCH_MEMORY*/
#endif

#if (CChans > 0)
    Float        AACD_buf_cc_coef[CChans][LN2];
    Float        AACD_buf_cc_gain[CChans][Chans][MAXBANDS];
    Float        AACD_buf_cc_state[CChans][LN2];
#endif
    long         AACD_bno;
    int          AACD_adts_header_present;
    ADTS_Header  AACD_adts_header;
    unsigned int AACD_SeekNeeded;
    MC_Info      *AACD_mip;
    Wnd_Shape    AACD_wnd_shape[Chans];
    byte         prev_bk[Chans];
    Info         AACD_only_long_info;
    Info         AACD_eight_short_info;
    Info         *AACD_winmap[NUM_WIN_SEQ];
    Info         *AACD_win_seq_info[NUM_WIN_SEQ];
    AACPD_ProgConfig   AACD_prog_config;
    int          AACD_adif_header_present;
    ADIF_Header  AACD_adif_header;
    Pulse_Info   AACD_pulse_info;
    short        AACD_sfbwidth128[(1 << LEN_MAX_SFBS)];
#ifndef USE_SCRATCH_MEMORY
    short        AACD_buf_factors[2][MAXBANDS];
#else
    short        *AACD_buf_factors[2];
#endif /*USE_SCRATCH_MEMORY*/
#if defined(MAIN) || defined (SSR)
    int          AACD_buf_lpflag[Chans][MAXBANDS];
    int          AACD_buf_prstflag[Chans][LEN_PRED_RSTGRP + 1];
    PRED_STATUS  AACD_buf_sp_status[Chans][LN];
    PRED_STATUS  *AACD_sp_status[Chans];
#endif

#ifdef LC
    int          AACD_buf_lpflag[Chans][1];
    int          AACD_buf_prstflag[Chans][1];
#endif
    TNS_frame_info AACD_buf_tns[2][1];

    Float        *AACD_coef[Chans];
    Lfract       *AACD_data[Chans];
    Lfract       *AACD_state[Chans];
    Lfract       *AACD_coef_lf[Chans];
    short        *AACD_factors[Chans];
    int          *AACD_lpflag[Chans];
    int          *AACD_prstflag[Chans];

    TNS_frame_info *AACD_tns[Chans];
    MC_Info      AACD_save_mc_info;
    MC_Info      AACD_mc_info;

#if (CChans > 0)
    Float        *AACD_cc_coef[CChans];
    Float        *AACD_cc_gain[CChans][Chans];
    byte         AACD_cc_wnd[CChans];
    Wnd_Shape    AACD_cc_wnd_shape[CChans];

#if (ICChans > 0)
    Float        *AACD_cc_state[ICChans];
#endif

#endif
    int          AACD_maxfac;
    int          AACD_default_config;
    int          AACD_current_program;

    int          AACD_first_cpe;

#ifdef USE_AIFF
    int      AACD_useAIFF = 0;
    unsigned short AACD_AIFF_channels = 1;
    unsigned short AACD_AIFF_wordsize = 16;
    unsigned long AACD_AIFF_frames = 0;
    unsigned long AACD_AIFF_sampleRate = 48000;
    MC_Info *AACD_AIFF_MC_Info = 0;
#endif

    int          nblocks;
    int          spacing;
    int          offset;
    int          j;
    int          j_end;
    Lfract       wc1;
    Lfract       ws1;
    Lfract       wc2;
    Lfract       ws2;
    Lfract       wc3;
    Lfract       ws3;
    Lfract       tempr;
    Lfract       tempi;
    Lfract       wstep_c;
    Lfract       wstep_s;

    unsigned int instance_id;
    int          offset_wordlen;
    int          decsymb_wordlen;
    int          bits_decoded;

    unsigned long bs_val;

#ifndef LC
    int          *tmp_lpflag;
#endif

    Lfract       *ci_p;
    Lfract       *pi_p;
    Lfract       *ci_p_prev;
    Lfract       *tmp_wind_ptr;
    Lfract       zero;

#ifdef EVAL_VERSION
    unsigned long AACD_EV_TotalSamples;
    unsigned long AACD_EV_MaxSamples;
#endif
#ifndef USE_SCRATCH_MEMORY
    char         AACD_buf_tns_struct[Chans][TNS_MAX_WIN *
                    (sizeof(TNSinfo) +
                    sizeof(TNSfilt) * TNS_MAX_FILT_SHORT +
                    sizeof(short) * TNS_MAX_FILT_SHORT * TNS_MAX_ORDER_SHORT)];
#else
    char        *AACD_buf_tns_struct[Chans];
#endif /*USE_SCRATCH_MEMORY*/
    unsigned char AACD_ibs_buf[INTERNAL_BS_BUFSIZE];
    byte         AACD_hasmask[Winds];
    byte         *AACD_mask[Winds];
    byte         *AACD_group[Chans];
    byte         AACD_wnd_prev[Chans];
    byte         AACD_max_sfb[Chans];
    byte         *AACD_cb_map[Chans];

    byte         AACD_buf_cb_map[2][MAXBANDS];
    byte         AACD_buf_group[2][NSHORT];
    byte         AACD_buf_mask[2][MAXBANDS];
    byte         AACD_wnd[Chans];
    byte         AACD_d_bytes[1];
    char         *out_ptr;
    AACPD_OutputFmtType *outbuf_ptr[Chans];
                          /* Format is short aacd_or long depending on 16
                             aacd_or 24 bit output */
    long         cur_noise_state;
    long         noise_state_save[MAXBANDS];
#ifndef LC
    int          lp_store[MAXBANDS];
#else
    int          lp_store[1];
#endif
    AACPD_Decoder_Config decoder_config;
    unsigned int num_pcm_bits;
    AACPD_Block_Params Params; /* This is an internal copy of parameter required
                                 to decode the current-block. Since some parameters
                                 change from block to block, this internal copy is
                                 required to detect the parameters which change.
                                 The change is detected by comparing the values in this
                                 copy with the values provided by the application,
                                 as part of AACPD_Decoder_Config.params. */

    unsigned int BitsInBlock;
    Lfract         *tbl_ptr_Coeff_FFTTwid_Cos;
#if defined(MOT_AAC) && !defined(AAC_ONLY)
    /* pointer to SBRD_global_struct */
    void *psSBRD_gbl_stru;
#endif
	SR_Info AACD_samp_rate_info[(1 << LEN_SAMP_IDX)];
	Lfract *AACD_windowPtr[2][2];
	unsigned char *AACD_huffstart[13];
	short *AACD_bitrev_indices[NUM_BLOCK_LENS];
}AACD_global_struct;
//tlsbo89610
#if defined(WINCE_WARNLVL4_DISABLE) && defined(__WINCE)

	// disable some level-4 warnings, use #pragma warning(enable:###) to re-enable

	#pragma warning(disable:4100) // warning C4100: unreferenced formal parameter

	#pragma warning(disable:4127) // warning C4127: conditional expression is constant

	//#pragma warning(disable:4189) // warning C4189: local variable is initialized but not referenced

	#pragma warning(disable:4201) // warning C4201: nonstandard extension used : nameless struct/union

	#pragma warning(disable:4214) // warning C4201: nonstandard extension used : bit field types other than int

	#pragma warning(disable:4244) // warning C4244: possible loss of data in conversion

	#pragma warning(disable:4310) // warning C4310: cast truncates constant value

	//#pragma warning(disable:4511) // warning C4511: copy constructor could not be generated

	//#pragma warning(disable:4512) // warning C4512: assignment operator could not be generated

	//#pragma warning(disable:4514) // warning C4514: unreferenced inline function has been removed

	#pragma warning(disable:4706) // warning C4706: assignment within conditional expression

	//#pragma warning(disable:4710) // warning C4710: 'function' not inlined

#endif //#if defined(WINCE_WARNLVL4_DISABLE) && defined(__WINCE)

#endif //_AAC_COMMON_H

