
 /************************************************************************
  * Copyright 2005-2006 by Freescale Semiconductor, Inc.
  * All modifications are confidential and proprietary information
  * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
  ************************************************************************/

/*******************, AACD_global_struct *ptr*********************************************************
 *
 *   CHANGE HISTORY
 *   dd/mm/yy   Code Ver     Description                        Author
 *   --------   -------      -----------                        ------
 *      --         --            --                               --
 *   13/09/04      02        Fixed Bug in SATURATE macro        S.Nachiappan
 *   18/04/07      03        Moved ARM_DCT_ASM and              Satish K Singh
 *                             ARM_TNS_ASM to makefile        
 *
 ****************************************************************************/


#ifndef _aac_fix_defs_h_
#define _aac_fix_defs_h_

/*----------------------------------------------------------------------------*
 * all.h
 *----------------------------------------------------------------------------*/
#ifndef UNREFERENCED_PARAMETER
#define UNREFERENCED_PARAMETER(P) (P)
#endif

#define NO_SCALING
#define USE_DCTIV_IMDCT

#define DCTIV_ARM_OPT


#define C_PI        3.14159265358979323846  /* pi */

/* prediction */
#define PRED_ORDER  2
#define PRED_ALPHA  0.90625
#define PRED_A      0.953125
#define PRED_B      0.953125

/* #define is required in order to use these args in #if () directive */
#define ICChans 0       /* independently switched coupling channels */
#define DCChans 0       /* dependently switched coupling channels */
#define XCChans 0       /* scratch space index for parsing unused coupling
                 * channels */
#define CChans  (ICChans + DCChans + XCChans)

#define MAX_BITS_PER_WORD 32

#define BLOCK_LEN_LONG   1024
#define BLOCK_LEN_SHORT  128
#define NFLAT       ((BLOCK_LEN_LONG - BLOCK_LEN_SHORT)/2)

#define ADTS_FRAME_MAX_SIZE 8192
#define ADTS_FRAME_HEADER_SIZE 7
/*----------------------------------------------------------------------------*
 * gc.h
 *----------------------------------------------------------------------------*/

/* This header file is for the gain control tool. The common include files,
defines aacd_and typedefs are declared. */

#ifdef  SSR
#define NBANDSBITS      2
#define NATKSBITS       3
#define IDGAINBITS      4
#define ATKLOCBITS      5
#define ATKLOCBITS_START_A  4
#define ATKLOCBITS_START_B  2
#define ATKLOCBITS_SHORT    2
#define ATKLOCBITS_STOP_A   4
#define ATKLOCBITS_STOP_B   5
#define NBANDS          4
#define NPQFTAPS        96
#define NPEPARTS        64  /* Num of PreEcho Inhibition Parts */

#define SHORT_WIN_IN_LONG   8
#define GC_BLOCKS_LONG  1
#define GC_BLOCKS_SHORT SHORT_WIN_IN_LONG
#define GC_BLOCKS_START 2
#define GC_BLOCKS_STOP  2

#define mylog2(x)       (mylog10(x)/mylog10(2))
#define mylog10(x)      ((((double)x)>1e-20) ? \
                log10((double)(x)) : log10((double)1e-20))
#define npow2(x)        (1L << (x)) /* 2^x */
#endif

/*----------------------------------------------------------------------------*
 * monopred.h
 *----------------------------------------------------------------------------*/

#define MAX_PGRAD   2
#define MINVAR      1
#define Q_ZERO      0x0000
#define Q_ONE       0x3F80

/*----------------------------------------------------------------------------*
 * stdinc.h
 *----------------------------------------------------------------------------*/

#ifndef _POSIX_SOURCE
#define                 _POSIX_SOURCE   /* stops repeat typdef of ulong */
#endif

//#define PRINT       fprintf
#define SO      stdout
#define SE      stderr
#define ARGBEGIN    for(argv++,argc--;\
                argv[0] && argv[0][0]=='-' && argv[0][1];\
                argc--, argv++) {\
                char *_args, *_argt;\
                char _argc;\
                _args = &argv[0][1];\
                if((_args[0]=='-') && (_args[1]==0)){\
                    argc--; argv++; break;\
                }\
                _argc = 0;\
                while((_argc = *_args++))\
                switch(_argc)
#define ARGEND      }
#define ARGF()      (_argt=_args, _args="",\
                (*_argt? _argt: argv[1]? (argc--, *++argv): 0))
#define ARGC()      _argc

/*----------------------------------------------------------------------------*
 * tns.h
 *----------------------------------------------------------------------------*/

#define TNS_MAX_BANDS       49
#define TNS_MAX_ORDER       31
#define TNS_MAX_ORDER_LONG  12
#define TNS_MAX_ORDER_SHORT 7
#define TNS_MAX_WIN         8
#define TNS_MAX_WIN_LONG    1
#define TNS_MAX_WIN_SHORT   8
#define TNS_MAX_FILT        3
#define TNS_MAX_FILT_LONG   3
#define TNS_MAX_FILT_SHORT  1
/* Fractional precision of reflection coefficients */
#define K_FRAC_PREC 31 
/* Fractional precision of AR coefficients */
#define AR_COEF_FRAC_PREC 26 

/*----------------------------------------------------------------------------*
 * fft.h
 *----------------------------------------------------------------------------*/
#define BASE 4

/*----------------------------------------------------------------------------*
 * imdct.h
 *----------------------------------------------------------------------------*/

#define NORM_TYPE 0
#define START_TYPE 1
#define SHORT_TYPE 2
#define STOP_TYPE 3

/*----------------------------------------------------------------------------*
 * pow_4by3.h
 *----------------------------------------------------------------------------*/

#define  POW_4BY3_TABLE_SIZE            8192
#define  POW_4BY3_MANTISSA_MASK         0xffffffe0 
#define  POW_4BY3_EXPONENT_MASK         0x0000001f
#define  POW_4BY3_NUM_MANTISSA_BITS     26
#define  DEQUANT_EXP_FOR_ZERO_MANTISSA -31
#define  LARGE_NEG_EXP                 -127
#define  MAX_EXP                        127

/*----------------------------------------------------------------------------*
 * scale_fac_tbl.h
 *----------------------------------------------------------------------------*/
#define  SCALE_FAC_TABLE_SIZE           5
#define  SCALE_NUM_FRACT_BITS           31

/* Bitstream related defines */
#define BIT_COUNTER_INIT 24
#define MIN_REQD_BITS 24
#define INTERNAL_BS_BUFSIZE 1024         //64

/* IMDCT */
#define IMDCT_FRACT_BITS 15
#define NUM_SAMPLES_PER_ITER 8

/*
Approximates the value to the nearest rounding integer.
It does the saturation if required.
*/
#define SATURATE(in,out,num_pcm_bits) \
        {\
            long tmp_long; \
            tmp_long = in; \
            tmp_long += (1 << (32 - num_pcm_bits - 2)); \
            tmp_long >>= (32 - num_pcm_bits - 2 + 1); \
            if (tmp_long > ( (1 << (num_pcm_bits - 1)) - 1)) \
                tmp_long = ( (1 << (num_pcm_bits - 1)) - 1); \
            else if (tmp_long < (-(1 << (num_pcm_bits-1)))) \
                tmp_long = -(1 << (num_pcm_bits-1)); \
            out = (short)(AACPD_OutputFmtType)tmp_long; \
        }
#ifdef EVAL_VERSION
#define EV_TIME_LIMIT 30    /* In seconds */
#endif

#endif              /* #ifndef _aac_fix_defs_h_ */
