
 /************************************************************************
  * Copyright 2005-2006 by Freescale Semiconductor, Inc.
  * All modifications are confidential and proprietary information
  * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
  ************************************************************************/

/****************************************************************************
 *
 *   CHANGE HISTORY
 *   dd/mm/yy   Code Ver     Description                        Author
 *   --------   -------      -----------                        ------
 *      --         --            --                               --
 *   19/08/04      01        Added LEN_EMPHASIS,                S.Nachiappan
 *                           for reading the 'Emphasis'
 *                           field in Old-format ADTS-Headers
 *                           in MPEG-4. This 'Emphasis' field
 *                           has been removed from the standard,
 *                           but some old-encoders put in this
 *                           field in the ADTS Header.
 *
 *                           Add 'tag' field in ProgConfig
 *                           Struct, since it is required to
 *                           process PCE's inside
 *                           AACD_decode_info
 *
 *                           Added struct AACPD_Block_Params
 *
 *  09/09/04       02        Moving structs ProgConfig          S.Nachiappan
 *                           MIXdown aacd_and EleList to
 *                           aacplus_dec_interface.h
 *  16/09/2004     03        fext in Ch_Info made static        S.Nachiapan
 *  29/06/2005     04        Updated for SBR integration        Ashok Kumar
 *  29/08/2005     05        Changed 5.1 channel configuration  Ashok Kumar
 *                           for stereo decoding
 ****************************************************************************/

#ifndef _aac_fix_types_h_
#define _aac_fix_types_h_

#include <stdio.h>
#ifndef ARM_OPT_MACROS
#include "fix.h"
#endif
#include "fix_cd.h"
#ifndef ARM_OPT_MACROS
#include "util2.h"
#endif

/*----------------------------------------------------------------------------*
 * all.h
 *----------------------------------------------------------------------------*/
#ifdef BIGDECODER
enum
{
    /*
     * channels for 5.1 main profile configuration
     * (modify for any desired decoder configuration)
     */
    FChans	= 15,	/* front channels: left, center, right */
    FCenter	= 1,	/* 1 if decoder has front center channel */
    SChans	= 18,	/* side channels: */
    BChans	= 15,	/* back channels: left surround, right surround */
    BCenter	= 1,	/* 1 if decoder has back center channel */
    LChans	= 1,	/* LFE channels */
    XChans	= 1,	/* scratch space for parsing unused channels */

    Chans	= FChans + SChans + BChans + LChans + XChans
};
#else
enum
{
    /*
     * channels for 5.1 main profile configuration
     * (modify for any desired decoder configuration)
     */
#if 1
    FChans	= 3,	/* front channels: left, center, right */
    FCenter	= 1,	/* 1 if decoder has front center channel */
    SChans	= 2,	/* side channels: */
    BChans	= 2,	/* back channels: left surround, right surround */
    BCenter	= 0,	/* 1 if decoder has back center channel */
    LChans	= 1,	/* LFE channels */
    XChans	= 1,	/* scratch space for parsing unused channels */
#else
    FChans	= 2,	/* front channels: left, center, right */
    FCenter	= 0,	/* 1 if decoder has front center channel */
    SChans	= 0,	/* side channels: */
    BChans	= 0,	/* back channels: left surround, right surround */
    BCenter	= 0,	/* 1 if decoder has back center channel */
    LChans	= 0,	/* LFE channels */
    XChans	= 0,	/* scratch space for parsing unused channels */
#endif
    Chans	= FChans + SChans + BChans + LChans + XChans
};
#endif


enum
{
    /* block switching */
    LN = 2048,
    SN = 256,
    LN2 = LN / 2,
    SN2 = SN / 2,
    LN4 = LN / 4,
    SN4 = SN / 4,
    NSHORT = LN / SN,
    MAX_SBK = NSHORT,

    ONLY_LONG_WINDOW = 0,
    LONG_START_WINDOW,
    EIGHT_SHORT_WINDOW,
    LONG_STOP_WINDOW,
    NUM_WIN_SEQ,

    WLONG = ONLY_LONG_WINDOW,
    WSTART,
    WSHORT,
    WSTOP,

    MAXBANDS = 16 * NSHORT, /* max number of scale factor bands */
    MAXFAC = 121,           /* maximum scale factor */
    MIDFAC = (MAXFAC - 1) / 2,
    SF_OFFSET = 100,        /* global gain must be positive */

    /* specify huffman tables as signed (1) aacd_or unsigned (0) */
    HUF1SGN = 1,
    HUF2SGN = 1,
    HUF3SGN = 0,
    HUF4SGN = 0,
    HUF5SGN = 1,
    HUF6SGN = 1,
    HUF7SGN = 0,
    HUF8SGN = 0,
    HUF9SGN = 0,
    HUF10SGN = 0,
    HUF11SGN = 0,

    ZERO_HCB = 0,
    BY4BOOKS = 4,
    ESCBOOK = 11,
    NSPECBOOKS = ESCBOOK + 1,
    BOOKSCL = NSPECBOOKS,
    NBOOKS = NSPECBOOKS + 1,
    INTENSITY_HCB2 = 14,
    INTENSITY_HCB = 15,

#ifndef MPEG4
    RESERVED_HCB = 13,
#else
    NOISE_HCB = 13,
    NOISE_PCM_BITS = 9,
    NOISE_PCM_OFFSET = (1 << (NOISE_PCM_BITS - 1)),
    NOISE_OFFSET = 90,
#endif
    LONG_SECT_BITS = 5,
    SHORT_SECT_BITS = 3,

    /* Program Configuration */
    Main_Profile = 0,
    LC_Profile = 1,
    SSR_Profile = 2,

    Fs_48 = 3,
    Fs_44 = 4,
    Fs_32 = 5,

    /* Misc constants */
    CC_DOM = 0,     /* before TNS */
    CC_IND = 1,

    /* Raw bitstream constants */
    LEN_SE_ID = 3,
    LEN_TAG = 4,
    LEN_COM_WIN = 1,
    LEN_ICS_RESERV = 1,
    LEN_WIN_SEQ = 2,
    LEN_WIN_SH = 1,
    LEN_MAX_SFBL = 6,
    LEN_MAX_SFBS = 4,
    LEN_CB = 4,
    LEN_SCL_PCM = 8,
    LEN_PRED_PRES = 1,
    LEN_PRED_RST = 1,
    LEN_PRED_RSTGRP = 5,
    LEN_PRED_ENAB = 1,
    LEN_MASK_PRES = 2,
    LEN_MASK = 1,
    LEN_PULSE_PRES = 1,
    LEN_TNS_PRES = 1,
    LEN_GAIN_PRES = 1,

    LEN_PULSE_NPULSE = 2,
    LEN_PULSE_ST_SFB = 6,
    LEN_PULSE_POFF = 5,
    LEN_PULSE_PAMP = 4,
    NUM_PULSE_LINES = 4,
    PULSE_OFFSET_AMP = 4,

    LEN_IND_CCE_FLG = 1,
    LEN_NCC = 3,
    LEN_IS_CPE = 1,
    LEN_CC_LR = 1,
    LEN_CC_DOM = 1,
    LEN_CC_SGN = 1,
    LEN_CCH_GES = 2,
    LEN_CCH_CGP = 1,

    LEN_D_ALIGN = 1,
    LEN_D_CNT = 8,
    LEN_D_ESC = 8,
    LEN_F_CNT = 4,
    LEN_F_ESC = 8,
    LEN_NIBBLE_MOT = 4,
    LEN_BYTE = 8,
    LEN_PAD_DATA = 8,

    LEN_PC_COMM = 8,

    /* FILL */
    LEN_EX_TYPE		= 4,	/* don't use 0x0 aacd_or 0xA as extension type! */
    EX_FILL		= 0,
    EX_FILL_DATA	= 1,
    EX_DRC		= 11,

    /* DRC */
    LEN_DRC_PL		= 7,
    LEN_DRC_PL_RESV	= 1,
    LEN_DRC_PCE_RESV	= (8 - LEN_TAG),
    LEN_DRC_BAND_INCR	= 4,
    LEN_DRC_BAND_RESV	= 4,
    LEN_DRC_BAND_TOP	= 8,
    LEN_DRC_SGN		= 1,
    LEN_DRC_MAG		= 7,
    DRC_BAND_MULT	= 4,	/* band top is multiple of this no of bins */

    /* sfb 40, coef 672, pred bw of 15.75 kHz at 48 kHz this is also the
     * highest number of bins used by predictor for any sampling rate */
    MAX_PRED_SFB = 40,  /* 48 kHz only, now obsolete */
    MAX_PRED_BINS = 672,

    ID_SCE = 0,
    ID_CPE,
    ID_CCE,
    ID_LFE,
    ID_DSE,
    ID_PCE,
    ID_FIL,
    ID_END,

    /* PLL's don't like idle channels! */
    FILL_VALUE = 0x55,

    /* program configuration element */
    LEN_PROFILE = 2,
    LEN_SAMP_IDX = 4,
    LEN_NUM_ELE = 4,
    LEN_NUM_LFE = 2,
    LEN_NUM_DAT = 3,
    LEN_NUM_CCE = 4,
    LEN_MIX_PRES = 1,
    LEN_MMIX_IDX = 2,
    LEN_PSUR_ENAB = 1,
    LEN_ELE_IS_CPE = 1,
    LEN_IND_SW_CCE = 1,
    LEN_COMMENT_BYTES = 8,

    /*
     * audio data transport stream frame format header
     */
	 /* fixed header */
	 LEN_SYNCWORD		= 12,
	 LEN_ID		= 1,
	 LEN_LAYER	= 2,
	 LEN_PROTECT_ABS	= 1,
	 LEN_PRIVTE_BIT	= 1,
	 LEN_CHANNEL_CONF	= 3,

	 /* variable header */
	 LEN_COPYRT_ID_ADTS		= 1,
	 LEN_COPYRT_START			= 1,
	 LEN_FRAME_LEN				= 13,
	 LEN_ADTS_BUF_FULLNESS	= 11,
	 LEN_NUM_OF_RDB			=2,

	 /* CRC */
	 LEN_CRC	= 16,

    /* Some encoders (PsyTEL) put an addtional field 'Emphasis'
       in MPEG-4 ADTS streams, though it a violation of the
       standard.
       But defining OLD_FORMAT_ADTS_HEADER will correctly decode
       such streams.
    */
#ifdef OLD_FORMAT_ADTS_HEADER
         LEN_EMPHASIS   =  2,
#endif

    /* audio data interchange format header */
    LEN_ADIF_ID = (32 / 8),
    LEN_COPYRT_PRES = 1,
    LEN_COPYRT_ID = (72 / 8),
    LEN_ORIG = 1,
    LEN_HOME = 1,
    LEN_BS_TYPE = 1,
    LEN_BIT_RATE = 23,
    LEN_NUM_PCE = 4,
    LEN_ADIF_BF = 20,
    XXX
};

enum
{
    AAC_ERROR_EOF = 0,
    AAC_ERROR_USAGE,
    AAC_ERROR_ADTS_HEADER,
    AAC_ERROR_PROG_CONFIG,
    AAC_ERROR_ENTER_CHN,
    AAC_ERROR_DEFAULT_POSITION,
    AAC_ERROR_MCINFO,
    AAC_ERROR_GETCC,
    AAC_ERROR_DATA_CHN,
    AAC_ERROR_START_BLOCK,
    AAC_ERROR_END_BLOCK,
    AAC_ERROR_ICS_INFO,
    AAC_ERROR_GET_MASK,
    AAC_ERROR_GETICS,
    AAC_ERROR_INITIO,
    AAC_ERROR_COUPLING,
    AAC_ERROR_PREDICT,
    AAC_ERROR_MEMORY,
    AAC_ERROR_HUFFDECODE,
    AAC_ERROR_GAIN_CONTROL,
    AAC_ERROR_PULSE_NC,
    AAC_ERROR_FILEIO,
    AAC_ERROR_DRC,
    AAC_ERROR_BLOCK_TYPE,
    AAC_ERROR_HEADER_TYPES,
    AAC_ERROR_TNS_COEF_RES,
    AAC_ERROR_FFT_LENGTH
};

typedef float   Float;
#if 0
#ifndef HAS_ULONG
typedef unsigned long ulong;

#endif
#endif
typedef unsigned char byte;


enum
{
    /* block switch windows for single channels aacd_or channel pairs */
    Winds = Chans,

    /* onyx_max length of DSE, bytes */
    MAX_DBYTES	= ((1<<LEN_D_CNT) + (1<<LEN_D_ESC)),

    TEXP = 128,     /* size of exp cache table */
    MAXFFT = LN4,

    XXXXX
};

typedef struct
{
    int             islong; /* true if long block */
    int             nsbk;   /* onyx_sub-blocks (SB) per block */
    int             bins_per_bk;    /* coef's per block */
    int             sfb_per_bk;     /* sfb per block */
    int             bins_per_sbk[MAX_SBK];  /* coef's per SB */
    int             sfb_per_sbk[MAX_SBK];   /* sfb per SB */
    int             sectbits[MAX_SBK];
    short          *sbk_sfb_top[MAX_SBK];   /* top coef per sfb per SB */
    short          *sfb_width_128;          /* sfb width for short blocks */
    short           bk_sfb_top[8 * 15];     /* cum version of above */
    int             num_groups;
    short           group_len[8];
    short           group_offs[8];
}               Info;

/*
typedef struct
{
    int             samp_rate;
    int             nsfb1024;
    short          *SFbands1024;
    int             nsfb128;
    short          *SFbands128;
}               SR_Info;
*/

typedef struct
{
    byte            this_bk;
}               Wnd_Shape;

typedef struct
{
    int             index;
    int             len;
    unsigned long           cw;
}               MOT_Huffman;

typedef struct
{
    int             index;
    unsigned long           cw;
}               HuffCodeWord;

typedef struct
{
    int             cwlen;
    int             ncw;
}               HuffLen;

typedef struct
{
    int             n;
    int             dim;
    int             lav;
    int             mod;
    int             off;
    int             signed_cb;
    HuffCodeWord   *huffcw;
    HuffLen        *hufflen;
    MOT_Huffman        *hcw;
}               Hcb;

/* used in de-interleave */
typedef int     DINT_DATATYPE;

#define MAX_FILE_EXTENION_LEN 6

typedef struct
{
    int             done;   /* channel is done */
    int             present;/* channel present */
    int             tag;    /* element tag */
    int             cpe;    /* 0 if single channel, 1 if channel pair */
    int             common_window;  /* 1 if common window for cpe */
    int             ch_is_left;     /* 1 if left channel of cpe */
    int             paired_ch;  /* index of paired channel in cpe */
    int             widx;       /* window element index for this channel */
    int             is_present; /* intensity stereo is used */
    int             ncch;       /* number of coupling channels for this ch */

#if (CChans > 0)
    int             cch[CChans];    /* coupling channel idx */
    int             cc_dom[CChans]; /* coupling channel domain */
    int             cc_ind[CChans]; /* independently switched coupling
                                     * channel flag */
#endif
    char           fext[MAX_FILE_EXTENION_LEN];        /* filename extension */
    int             file_opened; /* file is already open aacd_or not */
}               Ch_Info;

typedef struct
{
    int             nch;    /* total number of audio channels */
    int             nfsce;  /* number of front SCE's prior to first front CPE */
    int             prev_nfsce;  /* number of front SCE's prior to first front CPE */
    int             nfch;   /* number of front channels */
    int             nsch;   /* number of side channels */
    int             nbch;   /* number of back channels */
    int             nlch;   /* number of lfe channels */
    int             ncch;   /* number of valid coupling channels */
    int             cc_tag[(1<<LEN_TAG)];   /* tags of valid CCE's */
    int             cc_ind[(1<<LEN_TAG)];   /* independently switched CCE's */
    int             profile;
    int             sampling_rate_idx;
    Ch_Info         ch_info[Chans];
}               MC_Info;

typedef struct
{
    char            adif_id[LEN_ADIF_ID + 1];
    int             copy_id_present;
    char            copy_id[LEN_COPYRT_ID + 1];
    int             original_copy;
    int             home;
    int             bitstream_type;
    long            bitrate;
    int             num_pce;
    int             prog_tags[(1 << LEN_TAG)];
}               ADIF_Header;

typedef struct
{
    int             syncword;
    int             id;
    int             layer;
    int             protection_abs;
    int             profile;
    int             sampling_freq_idx;
    int             private_bit;
    int             channel_config;
    int             original_copy;
    int             home;
    int             copyright_id_bit;
    int             copyright_id_start;
    int             frame_length;
    int             adts_buffer_fullness;
    int             num_of_rdb;
    int             crc_check;
}               ADTS_Header;


/* The following structs ProgConfig, MIXdown, EleList aacd_and AACPD_Block_Params are
   defined in the Interface-file (aacplus_dec_interface.h) containing the API */
#ifdef AAC_ONLY
struct AACPD_MIXdown;
struct AACPD_EleList;
struct AACPD_ProgConfig;
struct AACPD_Block_Params;
#endif

#ifdef  SSR
typedef struct
{
    int             lev;
    int             loc;
}               GAIN_DATA;

typedef struct
{
    int             num_gain_data;
    GAIN_DATA       gData[7];
}               GAIN_INF;

typedef struct
{
    GAIN_INF        gBlock[8];
}               GAIN_BLOCK;

/* definition of filterbank control & info structure */

typedef struct
{
    int             max_band;
    GAIN_BLOCK      gInfo[4];
}               GAIN_CONTROL_DATA;

enum window_sequence
{
    ONLY_LONG_SEQUENCE = 0x0,
    LONG_START_SEQUENCE,
    EIGHT_SHORT_SEQUENCE,
    LONG_STOP_SEQUENCE
};

#endif

/* definition of gain info used by Sony. The gain info must be transmitted.
   The maximum number of bits used for gain_info is 66*4+2 bits per block
   per audio channel */

typedef struct
{
    int             lev;
    int             loc;
}               GDATA;

typedef struct
{
    int             num_gain_data;
    GDATA           gData[7];
}               GINFO;

typedef struct
{
    int             max_band;
    GINFO           gInfo[4];
}               GAIN_INFO;

/*----------------------------------------------------------------------------*
 * monopred.h
 *----------------------------------------------------------------------------*/
/* Predictor status information for mono predictor */
typedef struct
{
    short           r[MAX_PGRAD];   /* contents of delay elements */
    short           kor[MAX_PGRAD]; /* estimates of correlations */
    short           var[MAX_PGRAD]; /* estimates of variances */
}               PRED_STATUS;

/*----------------------------------------------------------------------------*
 * tns.h
 *----------------------------------------------------------------------------*/

typedef struct
{
    int             start_band;
    int             stop_band;
    int             order;
    int             direction;
    short          *coef;
}               TNSfilt;

typedef struct
{
    int             n_filt;
    int             coef_res;
    TNSfilt        *filt;
}               TNSinfo;

typedef struct
{
    int             n_subblocks;
    TNSinfo        *info;
}               TNS_frame_info;

/*
 * PULSE noiseless coding
 */
typedef struct
{
    int             pulse_data_present;
    int             number_pulse;
    int             pulse_start_sfb;
    int             pulse_position[NUM_PULSE_LINES];
    int             pulse_offset[NUM_PULSE_LINES];
    int             pulse_amp[NUM_PULSE_LINES];
}               Pulse_Info;

/* Bitstream parameters structure */
typedef struct
{
  long bit_counter;
  unsigned long bit_register;
  unsigned char *bs_curr;
  unsigned char *bs_end;
  unsigned char *bs_curr_ext;
  unsigned char *bs_end_ext;
  unsigned int bs_eof;
  unsigned int bs_seek_needed;
} BitstreamParam;


#endif              /* _aac_fix_types_h_ */
