
 /************************************************************************
  * Copyright 2005-2006 by Freescale Semiconductor, Inc.
  * All modifications are confidential and proprietary information
  * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
  ************************************************************************/

/*****************************************************************************
 *
 * Motorola Inc.
 * (c) Copyright 2005 Motorola, Inc.
 * ALL RIGHTS RESERVED.
 *
 *****************************************************************************
 * File Name: aacpd_dec_interface.h
 *
 * Description: aacplus decoder interface.
 *
 ****************************** Change History********************************
 *
 *   DD/MMM/YYYY     Code Ver     Description                   Author
 *   -----------     --------     -----------                   ------
 *   08/Jun/2005     0.1          File created                  Tommy Tang
 *   08/Jun/2005     0.2          redefine macro                Webber Wang
 *                                AACPD_MAX_NUM_MEM_REQS
 *                                add enum type AACPD_MEMALIGN.
 *                                add element aacd_mem_align in
 *                                structure AACPD_Mem_Alloc_Info_Sub
 *  20/Aug/2005      0.3          AACPD_OutputFmtType made as   Ganesh Kumar C
 *                                short/long depending on flag
 *                                "OUTPUT_24BITS"
 *  29/Aug/2005      1.0          Increased MAX_NUM_MEM_REQS by 1   Ashok Kumar
 *                                to add SBR decoder scratch and
 *                                added SBR decoder related error
 *                                return
 ******************************************************************************/

#ifndef _AACPD_DEC_INTERFACE_H
#define _AACPD_DEC_INTERFACE_H

#if defined(MOT_AAC) && !defined(AAC_ONLY)
#define AACP_FRAME_SIZE          2048
#else
#define AACP_FRAME_SIZE          1024
#endif
#define AACPD_TRUE               1
#define AACPD_NULL               ((void *)0)
#define AACPD_FAST_MEMORY        0
#define AACPD_SLOW_MEMORY        1

#if defined(MOT_AAC) && !defined(AAC_ONLY)
/* define this to use scratch buffer in SBR decoder */
#define SBR_USE_SCRATCH
#ifdef SBR_USE_SCRATCH
#define AACPD_MAX_NUM_MEM_REQS   5+1+1  /*For sbr decoder */
#else
#define AACPD_MAX_NUM_MEM_REQS   5+1  /*For sbr decoder */
#endif
#else
#define AACPD_MAX_NUM_MEM_REQS   5
#endif

#define AACPD_DATA_SIZE          0
#define AACPD_DATA_BEGIN         0       /* some known symbol */
#define AACPD_MEM_TYPE           AACPD_FAST_MEMORY /* can be changed */
#define AACPD_INPUT_BUFFER_SIZE  2048
#define AAC_FRAME_SIZE           1024  /* number of PCM samples per frame */

#define AACPD_NUM_TABLES         70    /* total number of tables used in
                                          aacPlus decoder */
#define AACPD_PATH_LEN           128
#define AACPD_LEN_TAG            4

/* Success is assigned to 0. As of now there can be 20
   warnings, starting from 11 to 30. Recoverable errors can be 20,
   starting from 31 to 50. Fatal errors can be 20, starting from 51 to 70.
   Later more error types can be added */

#define AACPD_WARNING_BASE       11
#define AACPD_RECOVERROR_BASE    31
#define AACPD_FATALERROR_BASE    51


#ifndef AACPD_UINT32
typedef unsigned int   AACPD_UINT32;
#endif
#ifndef AACPD_INT32
typedef int            AACPD_INT32;
#endif
#ifndef AACPD_UINT16
typedef unsigned short AACPD_UINT16;
#endif
#ifndef AACPD_INT16
typedef short          AACPD_INT16;
#endif
#ifndef AACPD_UCHAR
typedef unsigned char  AACPD_UCHAR;
#endif
#ifndef AACPD_UINT8
typedef unsigned char  AACPD_UINT8;
#endif
#ifndef AACPD_INT8
typedef char           AACPD_INT8;
#endif
#ifndef AACPD_CHAR
typedef char           AACPD_CHAR;
#endif
#ifndef AACPD_Void
typedef void           AACPD_Void;
#endif

#if defined(OUTPUT_24BITS) || defined(LONGBUFFER)
typedef signed long int AACPD_OutputFmtType;
#else
typedef signed short int AACPD_OutputFmtType;
#endif

/* enum with error return */
typedef enum
{
    AACPD_ERROR_NO_ERROR,
    AACPD_ERROR_EOF,
    /* Recoverable error */
    AACPD_ERROR_START_BLOCK=AACPD_RECOVERROR_BASE,
    AACPD_ERROR_HUFFDECODE,
    AACPD_ERROR_GETCC,
    AACPD_ERROR_DATA_CHN,
    AACPD_ERROR_PROG_CONFIG,
    AACPD_ERROR_MCINFO,
    AACPD_ERROR_END_BLOCK,
    AACPD_ERROR_HEADER_TYPES,
    AACPD_ERROR_NOT_SUPPORTED_PCM_BITS,
    AACPD_ERROR_NOT_SUPPORTED_CHANNEL_CONFIG,
    AACPD_ERROR_NOT_SUPPORTED_CHANNEL_COUNT,
    AACPD_ERROR_SEEKING,
    AACPD_ERROR_PNS,
    AACPD_ERROR_TNS_COEF_RES,
    AACPD_ERROR_BLOCK_TYPE,
    AACPD_ERROR_SBRD_CONCEAL,
    AACPD_ERROR_SBRD_NOT_INITIALIZED,
    /* Fatal error base*/
    AACPD_ERROR_INIT = AACPD_FATALERROR_BASE,
    AACPD_ERROR_INVALID_HEADER,
    AACPD_ERROR_INVALID_PARAMETERS,
    AACPD_ERROR_MEMALLOC
}AACPD_RET_TYPE;

/* enum to describe static and scratch memory */
typedef enum
{
    AACPD_MEM_SCRATCH = 0,
    AACPD_MEM_PERSISTENT
}AACPD_MEM_DESC;

/* enum to describe memory priority */
typedef enum
{
    AACPD_MEM_PRIORITY_HIGH = 0,
    AACPD_MEM_PRIORITY_LOW
}AACPD_MEM_PRIORITY;

/* enum to configure output precision */
typedef enum
{
    AACPD_16_BIT_OUTPUT = 16,
    AACPD_24_BIT_OUTPUT = 24
}AACPD_PCM_LENGTH;

#if defined(MOT_AAC) && !defined(AAC_ONLY)
/* enum for memory alignment requirement */
typedef enum
{
    AACPD_ALIGN_1_BYTE,
    AACPD_ALIGN_2_BYTE,
    AACPD_ALIGN_4_BYTE
} AACPD_MEMALIGN;
#endif

/* memory description structure */
typedef	 struct
{
    AACPD_INT32         aacpd_size;        /* Size in bytes */
    AACPD_INT32         aacpd_type;        /* Fast or Slow */
    AACPD_MEM_DESC      aacpd_mem_desc;    /* Static/Scratch memory*/
#if defined(MOT_AAC) && !defined(AAC_ONLY)
    AACPD_MEMALIGN      aacpd_mem_align;   /* required alignment */
#endif
    AACPD_MEM_PRIORITY  aacpd_priority;    /* In case of fast Memory type, specify the
                                                   priority */

    AACPD_Void          *app_base_ptr;     /* Pointer to the base memory,
		     			           which will be allocated and
                                                   filled by the application*/
} AACPD_Mem_Alloc_Info_Sub;


/* memory info structure */
typedef struct
{
    AACPD_INT32         aacpd_num_reqs;    /*  Number of valid memory requests */
    AACPD_Mem_Alloc_Info_Sub mem_info_sub[AACPD_MAX_NUM_MEM_REQS];
} AACPD_Mem_Alloc_Info;

/* decoder info structure, shall be filled by decoder */
typedef struct
{
    AACPD_UINT32	aacpd_sampling_frequency; /* Sampling frequency of the
                                                     current frame in Khz */
    AACPD_UINT32 	aacpd_num_channels;	  /* Number of channels decoded
                                                     in current frame */
    AACPD_UINT32    aacpd_bit_rate;
    AACPD_UINT32    aacpd_len;

    AACPD_UINT32	aacpd_frame_size;	  /* Number of stereo samples being
                                                     outputted for this frame */
    AACPD_INT8      output_path[AACPD_PATH_LEN];

    AACPD_UINT32    BitsInBlock;
    AACPD_INT8      sbrd_ps_present;        /* flag to indicate presence of
                                               PS related data */
} AACPD_Decoder_info;

/* describes number of audio elements in channels */
typedef struct
{
    AACPD_INT32         num_ele;
    AACPD_INT32         ele_is_cpe[(1 << AACPD_LEN_TAG)];
    AACPD_INT32         ele_tag[(1 << AACPD_LEN_TAG)];
}   AACPD_EleList;

/* structure to represent mixdown behavior of channels */
typedef struct
{
    AACPD_INT32         present;
    AACPD_INT32         ele_tag;
    AACPD_INT32         pseudo_enab;
}   AACPD_MIXdown;

/* program config structure, refer subpart 4
 * (description of program config element of [3] */
typedef struct
{
    AACPD_INT32         profile;            /* object type AAC-LC */
    AACPD_INT32         sampling_rate_idx;  /* sampling frequency index */
    AACPD_EleList       front;
    AACPD_EleList       side;
    AACPD_EleList       back;
    AACPD_EleList       lfe;
    AACPD_EleList       data;
    AACPD_EleList       coupling;
    AACPD_MIXdown       mono_mix;
    AACPD_MIXdown       stereo_mix;
    AACPD_MIXdown       matrix_mix;
    /* Ignore the comment field to minimize memory usage */
    AACPD_INT8          comments[1];
    AACPD_INT32         buffer_fullness;    /* put this transport level info here */

  /* Added the following the 'tag' field, to process ProgConfigElements outside
     of the header-parsing routines */

    AACPD_INT32         tag;
}   AACPD_ProgConfig;


/* structure contains member of ADIF/ADTS header block, please refer
[3] for detailed explanation of member variable of this structure */
typedef struct
{
    AACPD_INT32        num_pce;    /* number of program config element */
    AACPD_ProgConfig   *pce;

    AACPD_INT32        ChannelConfig;
    AACPD_INT32        SamplingFreqIndex;
    AACPD_INT32        BitstreamType;
    AACPD_INT32        BitRate;
    AACPD_INT32        BufferFullness;   /* No. of bits in encoder buffer after
                                            encoding first raw_data_block */
    AACPD_INT32        ProtectionAbsent;
    AACPD_INT32        CrcCheck;

  /* Please see FileHeader at the top for explanation */
#ifdef OLD_FORMAT_ADTS_HEADER
    AACPD_INT32        Flush_LEN_EMPHASIS_Bits;
#endif
} AACPD_Block_Params;


#if defined(MOT_AAC) && !defined(AAC_ONLY)
typedef struct
{
    AACPD_INT32    sbrd_down_sample;   /* flag to indicate SBR down sample mode */
    AACPD_INT32    sbrd_stereo_downmix;    /* flag related to stereo down mix */
}SBRD_Decoder_Config;
#endif

/* config structure of aacPlus decoder */
typedef struct AACPD_decoder_config_struct /* DSPhl28121*/
{
    AACPD_Mem_Alloc_Info    aacpd_mem_info;
    AACPD_Void       *aacpd_decode_info_struct_ptr;
    AACPD_UINT8      *aacd_initialized_data_start;
    /* Pointer to swap buffer function */
    AACPD_INT8       (*app_swap_buf)( AACPD_UCHAR **new_buf_ptr,
                               AACPD_UINT32 *new_buf_len,
                               struct AACPD_decoder_config_struct *dec_config);

    AACPD_UINT8      num_pcm_bits;  /* Specifies number of Bits in the
                                       PCM.(16 or 24)*/
    AACPD_Block_Params *params;

#if defined(MOT_AAC) && !defined(AAC_ONLY)
    SBRD_Decoder_Config sbrd_dec_config;
#endif
    void *pContext;
} AACPD_Decoder_Config;

#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN
#endif
/* Function prototypes */
EXTERN AACPD_RET_TYPE aacpd_query_dec_mem ( AACPD_Decoder_Config *dec_config);
/* decoder init function */
EXTERN AACPD_RET_TYPE aacpd_decoder_init ( AACPD_Decoder_Config *dec_config);
/* decode frame function */
EXTERN AACPD_RET_TYPE aacpd_decode_frame ( AACPD_Decoder_Config *dec_config,
                                    AACPD_Decoder_info *dec_info,
                                    AACPD_OutputFmtType out_buf[][AACP_FRAME_SIZE],
				    AACPD_INT8* buf, AACPD_INT32 buf_len);
/* swap buffer function, need to be implemented by application */
//AACPD_INT8 app_swap_buffers_aac_dec ( AACPD_UCHAR **new_buf_ptr,	//DSPhl28069 - name conflict defect
EXTERN AACPD_INT8 app_swap_buffers_aacplus_dec ( AACPD_UCHAR **new_buf_ptr,
                                      AACPD_UINT32 *new_buf_len,
                                      AACPD_Decoder_Config *dec_config);
/**************************<FUNCTION_PROTOTYPES END>***************************/

#endif  /* end of AACPD_DEC_INTERFACE_H header file */

/**************************<END OF THE FILE>***********************************/

