
 /************************************************************************
  * Copyright 2005-2006 by Freescale Semiconductor, Inc.
  * All modifications are confidential and proprietary information
  * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
  ************************************************************************/

/*******************************************************************************
 *
 * (C) 2001 MOTOROLA INDIA ELECTRONICS LTD.
 *
 *   CHANGE HISTORY
 *   dd/mm/yy   Code Ver     Description                        Author
 *   --------   -------      -----------                        ------
 *   27/07/99   00           Ver 0.01 of Onyx fix.h
 *   27/08/99   01           Enhanced for ColdFire              Nandakishore A.S
 *   03/09/99   02           Made asl generic (with/without     Darshak V
 *                           saturation) aacd_and added aacd_neg.
 *   09/09/99   03           Added the following functions:     K.Srinivasan
 *                             aacd_lsr, aacd_lsl, aacd_or & aacd_and.
 *   09/09/99   04           Made aacd_mpy_ll() a public function.   K.Srinivasan
 *                           Added aacd_test_val() function.
 *   11/01/00   05           Added Lfract differently for CF    K.Srinivasan
 *   24/06/00   06           Added aacd_cmp() function               B.Venkatarao
 *   12/07/00   07           Added aacd_get_acc_msbs() aacd_and           B.Venkatarao
 *                           aacd_lfract2long() functions
 *   13/07/00   08           Added aacd_lfract2shortint()            Bharath
 *   19/10/00   09           Removed add aacd_and sub function       P.Anandh
 *                              for compatablity.
 *   15/07/02   10           Make changes for cross compiling   Padmagowri P.
 *                           code on OnyxUL.
 *   04/03/04   11           Optimization aacd_and code change for   Sarang Akotkar
 *                           cross compiling on ARM 9E.
 ******************************************************************************/

#ifndef _fix_cd_h
#define _fix_cd_h

#ifndef UNREFERENCED_PARAMETER
#define UNREFERENCED_PARAMETER(P) (P)
#endif

#define LONG_TO_LFRACT(a) a

#if defined(ARM_INTEGRATION) || defined(ARM_OPT_MACROS)

#define FALSE 0
#define TRUE 1

typedef long int Word32;
typedef long int Lfract;
typedef short int Sfract;
typedef unsigned short int uSfract;

/* Defines */
#define SFRACT_PREC   (16)             /* Precision of Sfract */
#define EXT_PREC      (8)              /* Precision of Ext */
#define MS_EXT_PREC   (8)              /* Precision of More Significant Ext */
#define LS_EXT_PREC   (0)              /* Precision of Less Significant Ext */

#define WORD32_PREC (32)
#define LFRACT_PREC (2 * SFRACT_PREC)
#define ZERO ((Sfract)0)
#define MAXPOS32 ((Lfract)0x7fffffff)
#define MAXNEG32 ((Lfract)0x80000000)
#define MAXPOS16 ((Sfract)0x7fff)
#ifdef __cplusplus
#define MAXNEG16 ((Sfract)0x8000) /*remove warning c4310 in wince*/
#else
static const Sfract MAXNEG16 = 0x8000;
#endif

#define SFRACT_MASK (((Sfract) MAXNEG16) >> (SFRACT_PREC-1))
#define EXT_MASK (~((signed long int)MAXNEG32 >> (WORD32_PREC-EXT_PREC-1)))
#define EXT_LMASK (~((signed long int)MAXNEG32 >> (WORD32_PREC-EXT_PREC-1)))
#define MAX_POS_SFRACT MAXPOS16
#define MAX_POS_SFRACT_LO ((Sfract)MAXNEG16 >> (SFRACT_PREC - 1))
#define MAX_NEG_SFRACT ((Lfract)MAXNEG16)
#define MAX_NEG_SFRACT_LO ((Lfract)ZERO)
#define SFRACT_LSB ((Sfract) 1)
#define SFRACT_MSB ((Sfract)MAXNEG16)
#define EXT_LSB ((Lfract) 1)
#define EXT_MSB ((Lfract) 1 << EXT_PREC)
#define WORD32_MASK ((Lfract)MAXNEG32 >> (WORD32_PREC - 1))
#ifndef LFRACT_DEFINED
#define LFRACT_DEFINED
#define asl(a,b)  aacd_asl_generic(a,b,1)


/* Accumulator => long long :
    ------------------------------
   |           |  |               |
    ------------------------------
               <--><-------------->
                 8       32 bits
                Ext
               <-   Accumulator  ->
 */
typedef long long Acc; 

#define MAXNEG64 (((Acc) WORD32_MASK << WORD32_PREC) | MAXNEG32)

#endif

#else

//typedef enum {FALSE, TRUE} Flag;

#define FALSE 0
#define TRUE 1

/* Defines */
#define SFRACT_PREC   (16)             /* Precision of Sfract */
#define EXT_PREC      (8)              /* Precision of Ext */
#define MS_EXT_PREC   (8)              /* Precision of More Significant Ext */
#define LS_EXT_PREC   (8)              /* Precision of Less Significant Ext */

#define WORD32_PREC (32)
#define LFRACT_PREC (2 * SFRACT_PREC)
#define ZERO ((Sfract)0)
#define MAXPOS32 ((Sfract)0x7fffffff)
#define MAXNEG32 ((Sfract)0x80000000)
#define SFRACT_MASK ((Sfract)MAXNEG32 >> (SFRACT_PREC - 1))
#define EXT_MASK ((signed long int)MAXNEG32 >> (EXT_PREC - 1))
#define EXT_LMASK (~((signed long int)MAXNEG32 >> (WORD32_PREC-EXT_PREC-1)))
#define MAX_POS_SFRACT (((Sfract)MAXNEG32 >> (SFRACT_PREC - 1)) & MAXPOS32)
#define MAX_POS_SFRACT_LO ((Sfract)MAXNEG32 >> (SFRACT_PREC - 1))
#define MAX_NEG_SFRACT ((Sfract)MAXNEG32)
#define MAX_NEG_SFRACT_LO ((Sfract)ZERO)
#define SFRACT_LSB ((Sfract)1 << (WORD32_PREC - SFRACT_PREC))
#define SFRACT_MSB ((Sfract)MAXNEG32)
#define EXT_LSB ((Sfract)1 << (WORD32_PREC - EXT_PREC))
#define EXT_MSB ((Sfract)MAXNEG32)
#define WORD32_MASK ((Sfract)MAXNEG32 >> (WORD32_PREC - 1))
#ifndef LFRACT_DEFINED
#define LFRACT_DEFINED
#define asl(a,b)  aacd_asl_generic(a,b,1)
typedef long int Word32;
typedef long int Lfract;
typedef long int Sfract;
typedef unsigned long int uSfract;
typedef struct
{
    Sfract          ex;                /* Extension word */
    Sfract          hi;                /* Upper word, single precision fract */
    Sfract          lo;                /* Lower word, single precision fract */
} Acc;

#endif // 

#endif


/* Function prototypes needed by the user */
void sload (Sfract in, Acc *out);

#ifndef ARM_OPT_MACROS
void aacd_lload (Lfract in, Acc *out);
#endif

void sstore (Acc *in, Sfract *out);

#ifdef ARM_OPT_MACROS
void lstore (Acc *in, Lfract *out);
#endif

#ifdef ONYX_UL
void lstorex (Acc *in, Lfract _X *out);
#endif

#ifndef ARM_OPT_MACROS
void aacd_mvi2l (Word32 in, Lfract *out);
#endif

#ifdef ONYX_UL
void aacd_mvi2ly (Word32 in, Lfract _Y *out);
#endif

void mvi2s (Word32 in, Sfract *out);
void aacd_lfract2sfract(Lfract in,Sfract *out);
long aacd_norm (Acc *in);
void aacd_rnd (Acc *in);

#ifndef ARM_OPT_MACROS
void aacd_add_acc (Acc in1, Acc in2, Acc *out);
void aacd_sub_acc (Acc in1, Acc in2, Acc *out);
#endif

void aacd_rndc_l (Acc *in);

#ifndef ARM_OPT_MACROS
void aacd_asr_acc (Acc *in);
void aacd_asl_acc (Acc *in);
#endif

void aacd_sign_ext (Acc *out);

void aacd_mpy_ll_full(Lfract in1_lf,Lfract in2_lf,Acc *out);

#ifndef ARM_OPT_MACROS
void aacd_mac_ll (Lfract in1, Lfract in2, Acc *out);
void aacd_mac_sl (Sfract in1, Lfract in2, Acc *out);
#endif

void mac_ss (Sfract in1, Sfract in2, Acc *out);

#ifndef ARM_OPT_MACROS
void aacd_msac_ll (Lfract in1, Lfract in2, Acc *out);
#endif

void msac_sl (Sfract in1, Lfract in2, Acc *out);
void msac_ss (Sfract in1, Sfract in2, Acc *out);
void mpysu (Sfract ,uSfract ,Acc *in1); 
void aacd_mpy (Sfract , Sfract ,Acc *in1);
void aacd_add_acc (Acc in1, Acc in2, Acc *out);

#ifndef ARM_OPT_MACROS
void aacd_asr (Lfract *in, long count);
#endif

void aacd_asl_generic (Lfract *in, long count, short int sat_flag);
void add (Lfract in1, Lfract in2, Lfract *out);
void sub (Lfract in1, Lfract in2, Lfract *out);

//#ifndef ARM_OPT_MACROS
void aacd_neg (Lfract *in);
//#endif

#ifdef ONYX_UL
void negy (Lfract _Y *in);
#endif

void aacd_or (Lfract l1, Lfract l2, Lfract *out);
void aacd_and (Lfract l1, Lfract l2, Lfract *out);

void aacd_lsr (Lfract *in, long count);
void aacd_lsl (Lfract *in, long count);
long aacd_test_val (Lfract in);
long aacd_cmp (Lfract x, Lfract y);
Lfract aacd_get_acc_msbs (Acc a, long sh);

#ifndef ARM_OPT_MACROS
void aacd_mul16by16(Lfract in1, Lfract in2, Acc *out);
void aacd_mul16by32(Lfract in1, Lfract in2, Acc *out);
#endif

void mul16by16c(Lfract in1, Lfract in2, Acc *out);
void mul16by32c(Lfract in1, Lfract in2, Acc *out);
void aacd_macr_ss (Sfract in1, Sfract in2, Acc *out);
void aacd_mpy_sl (Sfract in1, Lfract in2, Acc *out);
void macsu_ss (Sfract in1, uSfract in2, Acc *out);
void aacd_impyl_ss (Word32 in1, Word32 in2, Acc *out);
void aacd_impyw_ss (Word32 in1, Word32 in2, Word32 *out);
void addw_ss (Sfract in1, Sfract in2, Sfract *out);
void subw_ss (Sfract in1, Sfract in2, Sfract *out);
#ifdef ONYX_UL
void asrr_lx (Lfract _X *in, short int count);
void asll_lx (Lfract _X *in, short int count);
#endif

#ifndef ARM_OPT_MACROS
void aacd_asrr_l (Lfract *in, short int count);
#endif

void aacd_asll_l (Lfract *in, short int count);
void asr16_l (Lfract *in);
void aacd_asr16_acc (Acc *in);
void aacd_asl16_l (Lfract *in);
void aacd_asl16_acc (Acc *in);
void lsr16_l (Lfract *in);
void aacd_lsr16_acc (Acc *in);
void aacd_clb_s (Sfract in, Sfract *count);
int aacd_clb_l (Lfract in);

#ifndef ARM_OPT_MACROS
void aacd_mpy_ll (Lfract in1, Lfract in2, Acc *out);
#endif

Sfract float2sfract (float x, long shft);
float sfract2float (Sfract x, long shft);
Lfract float2lfract (float x, long shft);
float lfract2float (Lfract x, long shft);
Lfract double2lfract (double x, long shft);
double lfract2double (Lfract x, long shft);
long aacd_lfract2long (Lfract in);
short int aacd_lfract2shortint (Lfract in);
void aacd_mvi2acc (Word32 in, Acc *out);
Lfract aacd_mpy_lstore (Lfract in1, Lfract in2);

#ifdef ARM_OPT_MACROS

/* Function macros for ARM */
#define aacd_mpy_ll(in1, in2, out) \
{\
     *out = ((((Acc)in1) * in2)+0x40000000) >> (LFRACT_PREC-1);\
}

#define aacd_asr_acc(in) \
{\
    *in >>= 1;\
}

#define aacd_asl_acc(in) \
{\
    *in <<= 1;\
}

#define aacd_lload(in, out) \
{\
    *out = (Acc) in;\
}

#define aacd_asrr_l(in, count) \
{\
    *in >>= count;\
}

#define aacd_mvi2l(in, out)\
{\
    *out = (long int)in;\
}

#define aacd_mac_ll(in1,in2,out)\
{\
    *out += ((((Acc)in1) * in2)+0x40000000) >> (LFRACT_PREC-1);\
}

#define aacd_msac_ll(in1, in2, out)\
{\
    *out -= (((Acc)in1) * in2) >> (LFRACT_PREC-1);\
}

#define aacd_mac_sl(in1, in2, out)\
{\
    *out += (((Acc)in1) * in2) >> (SFRACT_PREC-1);\
}

#define aacd_asr(in, count)\
{\
    if (count >= 32)\
        *in = 0; \
    else\
    (*in) = (Lfract)((*in) >> count);\
}

#define aacd_add_acc(in1, in2, out)\
{\
    *out = in1 + in2;\
}

#define aacd_sub_acc(in1, in2, out)\
{\
    *out = in1 - in2;\
}

#define add(in1, in2, out)\
{\
    *out = in1 + in2;\
}

#define sub(in1, in2, out)\
{\
    *out = in1 - in2;\
}

#define aacd_asl_generic(in, count, sat_flag)\
{\
    *in <<= count;\
}

#define aacd_mul16by32(in1, in2, out)\
{\
    *out += ((Acc) in1 * in2) >> (LFRACT_PREC-3);\
}

#if 0
#define lstore(in, out)\
{\
    Acc a;\
\
    a = *in;\
\
    /* Saturate if overflow */\
\
    if (a > MAXPOS32)\
    {\
        *out = (Lfract) MAXPOS32;\
    }\
    else if (a < ((Lfract) MAXNEG32))\
    {\
        *out = MAXNEG32;\
    }\
    else\
        *out = (Lfract) a;\
}
#endif

/* No overflow check aacd_and no saturation assuming that overflow
 * never happens.
 */
#if 1
#define lstore(in, out)\
{\
    *out = (Lfract) *in;\
}
#endif

#define aacd_lfract2int(lf) ((long int) lf)

/*__inline void aacd_neg (Lfract *in)
{
    *in = 0l - *in;
    return;
}*/

#define aacd_mul16by16(in1, in2, out)\
{\
    Lfract a;\
\
    in1 >>= (SFRACT_PREC-2);\
    in2 >>= SFRACT_PREC;\
\
    a = in1 * in2;\
\
    *out += (a << 1);\
}

#endif

int aacd_abs(int a);

#endif

/* End of file */
