/*=============================================================================

    Header Name:    mfw_gst_bufmgr.h

    General Description:    This Header file contains the code for direct render
                            abstract interface with GStreamer.

===============================================================================
                           Freescale Confidential Proprietary
                                   WMSG - Multimedia
                   (c) Copyright Freescale 2008, All Rights Reserved

Revision History:
                                Modification  Tracking
Author                          Date          Number     Description of Changes
---------------------           --------     ----------- ----------------------
Sario Hu(b01138)                03/10/2008      		 Initial Draft Version


Portability:    This code is written for Linux OS and Gstreamer

=============================================================================*/
#ifndef __MFW_GST_BUFMGR_H__
#define __MFW_GST_BUFMGR_H__


#ifndef DIRECT_RENDER_VERSION
#define DIRECT_RENDER_VERSION 2
#endif

//#define BM_DEBUG printf
#ifndef BM_FLOW
#define BM_FLOW(...)
#endif

#ifndef BM_TRACE_BUFFER
#define BM_TRACE_BUFFER(...)
#endif


/*=============================================================================
                                            MACROS
=============================================================================*/


#if (DIRECT_RENDER_VERSION==1)
/* Direct render v1, support get/reject decoder interface only */


#ifndef RENDER_BUFFER_RESERVE_NUM
#define RENDER_BUFFER_RESERVE_NUM 1
#endif

#ifndef RENDER_BUFFER_MAX_NUM
#define RENDER_BUFFER_MAX_NUM 5
#endif


typedef enum {
    BMDIRECT = 0,
	BMINDIRECT = 1
}BMMode;


static BMMode bm_mode = BMDIRECT;
static GSList * bm_list = NULL;
static int bm_localbuff_num = 0;
static int bm_localbuff_size = 0;

#define BM_CLEAN_LIST do{\
        while(bm_list){\
            gst_buffer_unref(bm_list->data);\
            (bm_list) = g_slist_remove((bm_list), (bm_list)->data);\
		};\
	}while(0)




#define BM_ALLCATE_RENDER_BUFFER(listbuf, tgtpad, size, pdata) do{\
		GstBuffer * buffer;\
		GstFlowReturn result;\
	    GstCaps *src_caps = NULL;\
		src_caps = GST_PAD_CAPS((tgtpad));\
		result = gst_pad_alloc_buffer_and_set_caps((tgtpad), 0,(size), src_caps,&buffer);\
		if (result==GST_FLOW_OK){\
			(pdata) = GST_BUFFER_DATA(buffer);\
			(listbuf)=g_slist_append((listbuf), buffer);\
			 BM_FLOW("BM_ALLCATE_RENDER_BUFFER %p\n", buffer);\
		}else{\
			(pdata) = NULL;\
			GST_ERROR("BM_ALLCATE_RENDER_BUFFER return failed!\n");\
		}\
	}while(0)

#define BM_FREE_RENDER_BUFFER(listbuf, pdata) do{\
		GSList * tmp = (listbuf);\
		GstBuffer * buffer;\
		while(tmp){\
			buffer = (GstBuffer *)(tmp->data);\
			if (GST_BUFFER_DATA(buffer)==(pdata)){\
				BM_FLOW("BM_FREE_RENDER_BUFFER %p\n", buffer);\
				(listbuf) = g_slist_remove((listbuf), buffer);\
				gst_buffer_unref(buffer);\
				break;\
			}\
			tmp = tmp->next;\
		}\
		if (tmp==NULL)\
			GST_ERROR("BM_FREE_RENDER_BUFFER buffer pointer isn't valid!\n");\
	}while (0)
	
#define BM_RENDER_RENDER_BUFFER(listbuf, pdata, tgtpad, status, timestamp, duration) do{\
		GSList * tmp = (listbuf);\
		GstBuffer * buffer;\
		while(tmp){\
			buffer = (GstBuffer *)(tmp->data);\
			if (GST_BUFFER_DATA(buffer)==(pdata)){\
				(listbuf) = g_slist_remove((listbuf), buffer);\
				BM_FLOW("BM_RENDER_RENDER_BUFFER %p\n", buffer);\
				GST_BUFFER_TIMESTAMP(buffer) = (timestamp);\
				GST_BUFFER_DURATION(buffer) = (duration);\
				status = gst_pad_push((tgtpad), buffer);\
				break;\
			}\
			tmp = tmp->next;\
		}\
		if (tmp==NULL)\
			GST_ERROR("BM_RENDER_RENDER_BUFFER buffer pointer isn't valid!\n");\
	}while (0)


#define BM_ALLCATE_LOCAL_BUFFER(listbuf, tgtpad, size, pdata) do{\
		GstBuffer * buffer;\
		while (bm_localbuff_num){\
			bm_localbuff_size = (size);\
			buffer = gst_buffer_new_and_alloc((size));\
			(listbuf)=g_slist_append((listbuf), buffer);\
			bm_localbuff_num --;\
		}\
		buffer = (listbuf)->data;\
		(listbuf) = g_slist_remove((listbuf), buffer);\
		(listbuf)=g_slist_append((listbuf), buffer);\
		pdata = GST_BUFFER_DATA(buffer);\
		BM_FLOW("BM_ALLCATE_LOCAL_BUFFER %p\n", buffer);\
	}while(0)

#define BM_FREE_LOCAL_BUFFER(listbuf, pdata) do{\
		BM_FLOW("BM_FREE_LOCAL_BUFFER %p\n", pdata);\
	}while (0)
	
#define BM_RENDER_LOCAL_BUFFER(listbuf, pdata, tgtpad, status, timestamp, duration) do{\
		GstBuffer * buffer;\
		GstFlowReturn result;\
	    GstCaps *src_caps = NULL;\
		src_caps = GST_PAD_CAPS((tgtpad));\
		result = gst_pad_alloc_buffer_and_set_caps((tgtpad), 0, bm_localbuff_size, src_caps,&buffer);\
		if (result==GST_FLOW_OK){\
			memcpy(GST_BUFFER_DATA(buffer), pdata, bm_localbuff_size);\
			GST_BUFFER_TIMESTAMP(buffer) = (timestamp);\
			GST_BUFFER_DURATION(buffer) = (duration);\
			BM_FLOW("BM_RENDER_LOCAL_BUFFER %p\n", buffer);\
			(status) = gst_pad_push((tgtpad), buffer);\
		}else{\
			(status) =  result;\
		}\
	}while (0)

	
/* General abstract interface for direct render v1 */

#define BM_INIT(rmdmode, decbufnum, rendbufnum) do{\
        BM_CLEAN_LIST;\
        BM_FLOW("BM init, buffer request codec:%d, render:%d.\n", decbufnum, rendbufnum);\
        bm_mode = (rmdmode);\
        if ((rmdmode)== BMDIRECT){\
            if ((decbufnum)+ RENDER_BUFFER_RESERVE_NUM > (rendbufnum))\
                bm_mode = BMINDIRECT;\
        }\
        if (bm_mode == BMINDIRECT){\
			BM_FLOW("BM run under indirect mode.\n");\
			bm_localbuff_num = decbufnum;\
		}else{\
			BM_FLOW("BM run under direct mode.\n");\
		}\
	}while(0)
        
        
#define BM_GET_BUFFER(tgtpad, size, pdata) do{\
		if (bm_mode==BMDIRECT){\
			BM_ALLCATE_RENDER_BUFFER(bm_list, tgtpad, size, pdata);\
		}else{\
			BM_ALLCATE_LOCAL_BUFFER(bm_list, tgtpad, size, pdata);\
		}\
	}while(0)

#define BM_REJECT_BUFFER(pdata) do{\
		if (bm_mode==BMDIRECT){\
			BM_FREE_RENDER_BUFFER(bm_list, pdata);\
		}else{\
			BM_FREE_LOCAL_BUFFER(bm_list, pdata);\
		}\
	}while (0)
	
#define BM_RENDER_BUFFER(pdata, tgtpad, status, timestamp, duration) do{\
	    if (bm_mode==BMDIRECT){\
			BM_RENDER_RENDER_BUFFER(bm_list, pdata, tgtpad, status, timestamp, duration);\
		}else{\
			BM_RENDER_LOCAL_BUFFER(bm_list, pdata, tgtpad, status, timestamp, duration);\
		}\
	}while (0)

#define BM_GET_MODE bm_mode


#elif (DIRECT_RENDER_VERSION==2)
/*Direct render v2, support get/release decoder interface only*/

typedef enum {
    BMDIRECT = 0,
	BMINDIRECT = 1
}BMMode;

#define BMFLAG (GST_BUFFER_FLAG_LAST>>1)

static BMMode bm_mode = BMDIRECT;
static GSList * bm_list = NULL;
static gint bm_buf_num = 0;

#define BM_CLEAN_LIST do{\
        while(bm_list){\
            if (GST_BUFFER_FLAG_IS_SET(bm_list->data, BMFLAG))\
                gst_buffer_unref(bm_list->data);\
            gst_buffer_unref(bm_list->data);\
            (bm_list) = g_slist_remove((bm_list), (bm_list)->data);\
		};\
	}while(0)

#define BM_INIT(rmdmode, decbufnum, rendbufnum) do{\
        BM_FLOW("BM_INIT\n");\
        bm_buf_num = decbufnum;\
        BM_CLEAN_LIST;\
	}while(0)
        
        
#define BM_GET_BUFFER(tgtpad, size, pdata) do{\
        GstBuffer * buffer;\
		GstFlowReturn result;\
	    GstCaps *src_caps = NULL;\
		src_caps = GST_PAD_CAPS((tgtpad));\
		result = gst_pad_alloc_buffer_and_set_caps((tgtpad), 0,(size), src_caps,&buffer);\
		if (result==GST_FLOW_OK){\
		    GST_BUFFER_FLAG_SET(buffer, BMFLAG);\
			(pdata) = GST_BUFFER_DATA(buffer);\
			gst_buffer_ref(buffer);\
			bm_list=g_slist_append(bm_list, buffer);\
			BM_FLOW("BM_GET_BUFFERv2 %p:d%p\n", buffer, pdata);\
			BM_TRACE_BUFFER("codec request %p:d%p\n", buffer, pdata);\
			break;\
		}\
    	if (result!=GST_FLOW_OK){\
            (pdata)=NULL;\
            g_print("BM_GET_BUFFERv2 no buffer, %d in codec\n", g_slist_length(bm_list));\
    	}\
	}while(0)

#define BM_QUERY_HWADDR(pdata, hwbuffer) do{\
		GSList * tmp = (bm_list);\
		GstBuffer * buffer;\
		while(tmp){\
			buffer = (GstBuffer *)(tmp->data);\
			if (GST_BUFFER_DATA(buffer)==(pdata)){\
                (hwbuffer) = GST_BUFFER_OFFSET(buffer);\
				BM_FLOW("BM_HWTRANSITION v%p=h%p\n", buffer, (hwbuffer));\
				break;\
			}\
			tmp = tmp->next;\
		}\
		if (tmp==NULL)\
			g_print("BM_HWTRANSITION illegal %p!\n", pdata);\    
	}while (0)
      

#define BM_RELEASE_BUFFER(pdata) do{\
		GSList * tmp = (bm_list);\
		GstBuffer * buffer;\
		while(tmp){\
			buffer = (GstBuffer *)(tmp->data);\
			if (GST_BUFFER_DATA(buffer)==(pdata)){\
				BM_FLOW("BM_RELEASE_BUFFERv2 %p:d%p\n", buffer, pdata);\
				if (GST_BUFFER_FLAG_IS_SET(buffer, BMFLAG)){\
                    GST_BUFFER_FLAG_UNSET(buffer, BMFLAG);\
                }else{\
				    (bm_list) = g_slist_remove((bm_list), buffer);\
				    BM_TRACE_BUFFER("codec release %p:d%p\n", buffer, pdata);\
                }\
				gst_buffer_unref(buffer);\
				break;\
			}\
			tmp = tmp->next;\
		}\
		if (tmp==NULL)\
			g_print("BM_RELEASE_BUFFERv2 illegal %p!\n", pdata);\    
	}while (0)

#define BM_REJECT_BUFFER(pdata) do{\
		GSList * tmp = (bm_list);\
		GstBuffer * buffer;\
		g_print("BM_REJECT_BUFFER shuold not here %p!\n", pdata);\
		while(tmp){\
			buffer = (GstBuffer *)(tmp->data);\
			if (GST_BUFFER_DATA(buffer)==(pdata)){\
				BM_FLOW("BM_REJECT_BUFFERv2 %p:d%p\n", buffer, pdata);\
				if (GST_BUFFER_FLAG_IS_SET(buffer, BMFLAG)){\
                    GST_BUFFER_FLAG_UNSET(buffer, BMFLAG);\
                }else{\
				    (bm_list) = g_slist_remove((bm_list), buffer);\
				    BM_TRACE_BUFFER("codec release %p:d%p\n", buffer, pdata);\
				}\
				gst_buffer_unref(buffer);\
				break;\
			}\
			tmp = tmp->next;\
		}\
		if (tmp==NULL)\
			g_print("BM_RELEASE_BUFFERv2 illegal %p!\n", pdata);\    
	}while (0)

#define BM_RENDER_BUFFER(pdata, tgtpad, status, timestamp, duration) do{\
        GSList * tmp = (bm_list);\
		GstBuffer * buffer;\
		while(tmp){\
			buffer = (GstBuffer *)(tmp->data);\
			if (GST_BUFFER_DATA(buffer)==(pdata)){\
				BM_FLOW("BM_RENDER_BUFFERv2 %p:d%p\n", buffer, pdata);\
				if (GST_BUFFER_FLAG_IS_SET(buffer, BMFLAG)){\
                    GST_BUFFER_FLAG_UNSET(buffer, BMFLAG);\
                }else{\
				    (bm_list) = g_slist_remove((bm_list), buffer);\
				    BM_TRACE_BUFFER("codec release %p:d%p\n", buffer, pdata);\
				}\
				GST_BUFFER_TIMESTAMP(buffer) = (timestamp);\
				GST_BUFFER_DURATION(buffer) = (duration);\
				status = gst_pad_push((tgtpad), buffer);\
				break;\
			}\
			tmp = tmp->next;\
		}\
		if (tmp==NULL)\
			g_print("BM_RENDER_BUFFERv2 illegal %p!\n", pdata);\	
	}while (0)

#define BM_GET_MODE bm_mode
#define BM_GET_BUFFERNUM bm_buf_num

#endif	
#endif//__MFW_GST_BUFMGR_H__			

