
 /************************************************************************
  * Copyright 2005-2007 by Freescale Semiconductor, Inc.
  * All modifications are confidential and proprietary information
  * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
  ************************************************************************/

/***************************************************************************
* Property of Motorola, Motorola India Electronics Pvt Limited
****************************************************************************
* ANSI C source code
*
* Project Name : MP3 Decoder
*
* MOTOROLA CONFIDENTIAL PROPRIETARY
***************************************************************************/
/***************************************************************************
 *
 (C) 2001 MOTOROLA INDIA ELECTRONICS LTD.
 *
 *   CHANGE HISTORY
 *   dd/mm/yy   Code Ver     Description                   Author
 *   --------   -------      -----------                   ------
 *   16 APR 04    0.1 		created file                   Mahesh D.S
 *   17 AUG 04    0.2       Changes for ARM11 release      Mahesh D.S
 *   24 AUG 04    0.3       Changes for seperate scratch   mahesh D.S
 **************************************************************************/
/*********************************************************************************************
 *		Freescale Semiconductor, Inc.
 *		=============================
 *   CHANGE HISTORY
 *   dd/mm/yy   Code Ver     Description						  			Author
 *   --------   -------      -----------									------
 *   05 Apr 07	1.0			Added Layer info and modified size of			Madhav Varma
 *							Global Structure to support MPEG1-Layer2
 *********************************************************************************************/
#ifndef __MP3D_DEC_INTERFACE_H_
#define __MP3D_DEC_INTERFACE_H_

#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN 
#endif

#define MP3D_TRUE	                      1
#define MP3D_INPUT_BUF_SIZE               2048
#define MP3D_FAST_MEMORY                  1
#define MP3D_SLOW_MEMORY                  2
#define MP3D_MAX_NUM_MEM_REQS             2 /* presently we need seperate chunk
											   of memory for scratch_lf */
#define MP3D_BEGIN_INITIALIZED_DATA       Cs_Ca_tbl_lf
#define MP3D_DATA_SIZE                    /* presently not used */
#define MP3D_DATA_ALIGN_IN_BYTES          4
#define MP3D_MEM_TYPE                     FAST_MEMORY /* can be changed */
#define MP3D_INPUT_BUFFER_SIZE            2048
#define MP3D_FRAME_SIZE                   576   /* No of stereo samples outputed per 
		                                            call to the decode frame */

#ifndef MPEGLAYER2
#ifdef MP3D_HUF_DEQ_BYTEBUF_OPT_C
#define MP3D_GLOBAL_STRUCT_SIZE	      0x390
#else
#define MP3D_GLOBAL_STRUCT_SIZE	          0x6a0
#endif
#else
#ifdef MPEGLAYER1
#ifdef MP3D_HUF_DEQ_BYTEBUF_OPT_C
#ifdef MP2_QMFS_OPT_C
#define MP3D_GLOBAL_STRUCT_SIZE         (0x578 + 0x8+0x114c)
#else
#define MP3D_GLOBAL_STRUCT_SIZE         (0x2580+0x114c)
#endif
#else
#define MP3D_GLOBAL_STRUCT_SIZE	          (0x2aa0+0x114c)
#endif
#else
#ifdef MP3D_HUF_DEQ_BYTEBUF_OPT_C
#ifdef MP2_QMFS_OPT_C
#define MP3D_GLOBAL_STRUCT_SIZE         (0x578 + 0x8)
#else
#define MP3D_GLOBAL_STRUCT_SIZE         (0x2580)
#endif
#else
#define MP3D_GLOBAL_STRUCT_SIZE	          (0x2aa0)
#endif
#endif
#endif

#ifdef MP3D_QMF_WINDOW_SIMD_OPT_ASM            
#define MP3D_TOTAL_BUFFERS_SIZE           0x2500 
#else
#define MP3D_TOTAL_BUFFERS_SIZE           0x3500 
#endif

#define MP3D_SCRATCH_SIZE    			  0x1200 /* Scratch size in bytes */

/* Data type */
typedef unsigned char     MP3D_UINT8;
typedef char              MP3D_INT8;
typedef unsigned short    MP3D_UINT16;
typedef short             MP3D_INT16;
typedef unsigned int      MP3D_UINT32;
typedef int               MP3D_INT32;

/* Error types */
typedef enum 
{
  /* Sucess */
  MP3D_OK = 0,			   /* no error 						*/
  /* Warning : End of bitstream */
  MP3D_END_OF_STREAM,      /* end of bit stream 			*/
  /* Recoverable Errors */
  MP3D_ERROR_LAYER = 31,   /* invalid layer 				*/
  MP3D_ERROR_SAMP_FREQ,    /* invalid sampling frequency 	*/
  MP3D_ERROR_BIT_RATE,     /* invalid bit rate 				*/ 
  MP3D_ERROR_BLOCK_TYPE,   /* invalid block type 			*/ 
  MP3D_ERROR_CRC,          /* CRC error 					*/
  MP3D_ERROR_STREAM,
  MP3D_ERROR_DATA,         /* not enough main data 			*/ 
  MP3D_ERROR_WRAP,         /* bit error detected in hwrapbuf*/
  MP3D_ERROR_FREE_BIT_RATE,/* error computing free bit rate	*/  
  MP3D_ERROR_HUFFMAN_TABLE,/* Selection of wronge huffman table */
  MP3D_ERROR_VERSION,      /* Wrong version */
  /* Fatal Error */
  MP3D_ERROR_INIT = 51,    /* initialization error 			*/
  MP3D_ERROR_INVALID_PARAM = 52,
  /* Invalid Parameters to decode frame */
} MP3D_RET_TYPE;

typedef enum
{
    MP3D_16_BIT_OUTPUT,     /* 16-bit output format */
    MP3D_24_BIT_OUTPUT      /* 24-bit output format */
} MP3D_OUTPUT_FORMAT;

typedef enum
{
    MP3D_STATIC_MEM,        /* 0 for static memory */
    MP3D_SCRATCH_MEM        /* 1 for scratch memory */
} MP3D_MEM_DESC;

typedef struct {
	MP3D_INT32		mp3d_size;     /* Size in bytes */
	MP3D_INT32 	    mp3d_type;	   /* Memory type Fast or Slow */
	MP3D_MEM_DESC   mp3d_mem_desc; /* to indicate if it is scratch memory */
	MP3D_INT32      mp3d_priority; /* In case of fast memory, specify the priority */
	void 	        *app_base_ptr; /* Pointer to the base memory , which will be allocated and 
                                    * filled by the  application */
} MP3D_Mem_Alloc_Info_Sub;

typedef struct {
	MP3D_INT32 	mp3d_sampling_freq; /* Sampling frequency of the current frame in Khz */
	MP3D_INT32 	mp3d_num_channels;  /* Number of channels decoded in current frame */
	MP3D_INT32 	mp3d_frame_size;    /* Number of stereo samples being outputted for this  
                                     * frame */
	MP3D_INT32 mp3d_bit_rate;		/* tlsbo79740 - bit-rate information*/
	MP3D_INT32 layer;				/* engr30582 - MPEG layer of the current stream 
									 * 1 for layer-I,2 for layer-II and 3 for layer-III */
} MP3D_Decode_Params;

typedef struct {
	MP3D_INT32 				mp3d_num_reqs;
	MP3D_Mem_Alloc_Info_Sub	mem_info_sub [MP3D_MAX_NUM_MEM_REQS];
} MP3D_Mem_Alloc_Info;


struct MP3D_Dec_Config;
typedef struct MP3D_Dec_Config {
	MP3D_Mem_Alloc_Info		mp3d_mem_info;
	void					*mp3d_decode_info_struct_ptr; // Global_struct
//DSPhl28081 Removed the variable 
//MP3D_UINT8              *app_initialized_data_start;
	MP3D_OUTPUT_FORMAT      app_out_format;
    MP3D_INT8              (*app_swap_buf) (MP3D_UINT8 ** new_buf_ptr,
                                            MP3D_INT32 * new_buf_len,
                                            struct MP3D_Dec_Config *dec_config);
                            /* Pointer to swap buffer function */
} MP3D_Decode_Config;

#ifdef __SYMBIAN32__
#define IMPORT_C __declspec(dllimport)
#define EXPORT_C __declspec(dllexport)
#define EXTERN
#else
#define EXPORT_C
#define IMPORT_C
#endif
IMPORT_C EXTERN MP3D_RET_TYPE mp3d_query_dec_mem (MP3D_Decode_Config *dec_config);

#ifdef MP3D_HUF_DEQ_BYTEBUF_OPT_C
IMPORT_C EXTERN MP3D_RET_TYPE mp3d_decode_init (MP3D_Decode_Config *dec_config, MP3D_UINT8 *input_buffer, 
                                MP3D_INT32 input_buffer_length);
#else
IMPORT_C EXTERN MP3D_RET_TYPE mp3d_decode_init (MP3D_Decode_Config *dec_config, MP3D_INT16 *input_buffer, 
                                MP3D_INT32 input_buffer_length);
#endif
IMPORT_C EXTERN MP3D_RET_TYPE mp3d_decode_frame  (MP3D_Decode_Config *dec_config,
                                  MP3D_Decode_Params *dec_param, MP3D_INT32 *out_buf);

#ifdef MP3D_HUF_DEQ_BYTEBUF_OPT_C
IMPORT_C MP3D_INT8 app_swap_buffers_mp3_dec(MP3D_UINT8 **new_buf_ptr, MP3D_INT32 *new_buf_size,
                                    MP3D_Decode_Config *dec_config);
#else
IMPORT_C MP3D_INT8 app_swap_buffers_mp3_dec(MP3D_INT16 **new_buf_ptr, MP3D_INT32 *new_buf_size,
                                    MP3D_Decode_Config *dec_config);
#endif
#endif
