
 /************************************************************************
  * Copyright 2005-2006 by Freescale Semiconductor, Inc.
  * All modifications are confidential and proprietary information
  * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
  ************************************************************************/


#ifndef DECODERINTERFACE_H
#define DECODERINTERFACE_H

/*
*
********************************************************************************
*
* Copyright 2005-2006 by Freescale Semiconductor, Inc.
* All modifications are confidential and proprietary information
* of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
*
*******************************************************************************
*
*  File Description:   Contains header info for decoder interface.
*
*    DD/MMM/YYYY   Code Ver      Description                 Author
*    -----------   --------      -----------              ------------
*    12/Jun/2006     01           Created       MANOJ ARVIND & DURGAPRASAD S. BILAGI
*                    
*******************************************************************************
*
*/
 

#define MAX_NUM_MEM_REQS 30         /*! Maximum Number of MEM Requests  */

/*! All the API return one of the following */

typedef enum
{
    /* Successfull return values */
    E_MPEG2D_SUCCESS = 0,         /*!< Success                             */
    E_MPEG2D_PARTIAL_DECODE,
    E_MPEG2D_ENDOF_BITSTREAM,     /*!< End of Bit Stream                   */
    E_MPEG2D_FRAME_READY,
    /* Successful return with warning, decoding can continue */
    /* Start with number 11 */
    
    /* recoverable error return, correct the situation and continue */
    E_MPEG2D_NOT_ENOUGH_BITS = 31,/*!< Not enough bits are provided        */
    E_MPEG2D_OUT_OF_MEMORY,       /*!< Out of Memory                       */
    E_MPEG2D_WRONG_ALIGNMENT,     /*!< Incorrect Memory alignment          */
    E_MPEG2D_SIZE_CHANGED,        /*!< Image size changed                  */
    E_MPEG2D_INVALID_ARGUMENTS,   /*!< API arguments are invalid           */

    /* irrecoverable error type */
    E_MPEG2D_ERROR_STREAM = 51,   /*!< Errored Bitstream                   */
    E_MPEG2D_FAILURE,             /*!< Failure                             */
    E_MPEG2D_UNSUPPORTED,         /*!< Unsupported Format                  */
    E_MPEG2D_NO_IFRAME,           /*!< MPEG2D_first frame is not an I frame       */
    E_MPEG2D_SIZE_NOT_FOUND,      /*!< Frame size not found in bitstream   */
    E_MPEG2D_NOT_INITIALIZED      /*!< Decoder Not Initialised             */
    
} eMpeg2DecRetType;


/*! \enum Specifies the current functional state of the Decoder */
typedef enum
{
    E_MPEG2D_INVALID = 0,        /*!< Invalid Decoder State */
    E_MPEG2D_PLAY,               /*!< Decoder is decoding frames */
    E_MPEG2D_FF,                 /*!< Decoder is skipping frames */
    E_MPEG2D_REW                 /*!< Decoder is skipping frames in a direction
                                      opposite to PLAY and FF. Current Decoder
                                      doesn't support REW feature */
} eDecodeState;

/*! \enum Enumeration of possible buffer alignment */
typedef enum
{
    E_MPEG2D_ALIGN_NONE = 0,  /*!< buffer can start at any place           */
    E_MPEG2D_ALIGN_HALF_WORD, /*!< start address's last bit has to be 0    */
    E_MPEG2D_ALIGN_WORD       /*!< start adresse's last 2 bits has to be 0 */
} eMpeg2DecMemAlignType;



/*! Different structure definitions */

/*! structure for Y Cb Cr output from the decoder. */
typedef struct
{
    unsigned char   *pu8YBuf;     /*!< Y Buf       */
    int             s32YBufLen;   /*!< Y Buf Len   */
} sMpeg2DecYCbCrBuffer;


/*! Structure to hold each memory block requests from the decoder.
 *  The size and alignment are must to meet criteria, whereas others
 *  help to achive the performance projected. 
 */

typedef struct
{
    int 	s32Size;         /*!< size of the memory block            */
    int 	s32Type;         /*!< type of the memory - slow/fast and
                                  static/scratch                      */
    int     s32Priority;     /*!< how important the block is          */
    int 	s32Align;        /*!< alignment of the memory block       */
    void 	*pvBuffer;       /*!< pointer to allocated memory buffer  */
} sMpeg2DecMemBlock;


/*! Structure to hold all the memory requests from the decoder  */

typedef struct
{
    int               s32NumReqs;                   /*!< Number of blocks  */
	int 			  s32BlkNum ;					/*!< block number  */
    sMpeg2DecMemBlock asMemBlks[MAX_NUM_MEM_REQS];  /*!< array of requests */
} sMpeg2DecMemAllocInfo;


/*! Structure for defining the decoder output. */
typedef struct
{
    sMpeg2DecYCbCrBuffer sOutputBuffer;           /*!< Decoded frame      */
    signed char         *p8MbQuants;              /*!< Quant values       */       //ask
    unsigned short int 	 u16FrameWidth;           /*!< FrameWidth         */
    unsigned short int 	 u16FrameHeight;          /*!< FrameHeight        */
    unsigned int         bitrate;                 /*!< Bitrate			  */
    
    
} sMpeg2DecoderParams;



/*! This structure defines the decoder context. All the decoder API
 *  needs this structure as one of its argument.
 */
typedef struct
{
    sMpeg2DecMemAllocInfo   sMemInfo;      /*!< memory requirements info */
    sMpeg2DecoderParams     sDecParam;     /*!< decoder parameters       */
    void                    *pvMpeg2Obj;   /*!< decoder library object   */
    void                    *pvAppContext; /*!< Anything app specific    */
    eDecodeState            eState;        /*!< Indicates current Decoder
                                                State */
	/*Added for call back changes*/

    int (* ptr_cbkMPEG2DBufRead) (int *, unsigned char **, int , void *);

    /*Added for call back changes*/

   
} sMpeg2DecObject;



//Function Declarations

/*! Function to query memory requrement of the decoder */
eMpeg2DecRetType  eMPEG2DQuerymem(sMpeg2DecObject *psMp2Obj);
                                 
/*! Function to query memory requrement of the decoder */
eMpeg2DecRetType  eMPEG2D_Re_Querymem(sMpeg2DecObject *psMp2Obj);

/*! Function to initialize the decoder */
eMpeg2DecRetType  eMPEG2D_Init (sMpeg2DecObject *psMp2Obj);

/*! API to decode single frame of the encoded bitstream */
extern eMpeg2DecRetType eMPEG2Decode(sMpeg2DecObject *psMp2Obj,unsigned int *s32decodedBytes, void *pvAppContext);

/*! Function to free the resources allocated by decoder, if any */
eMpeg2DecRetType  eMPEG2DFree (sMpeg2DecObject *psMp2Obj);


eMpeg2DecRetType  eMPEG2D_ReInit (sMpeg2DecObject *psMp2Obj);

/*! \brief
 * Call back function for reading the input buffer by the decoder.
 *
 * Whenver the internul buffer inside the decoder is empty, the decoder will
 * call this function, passing the application context also. The application
 * shall implement a function to provide the required portion of the bitstream
 * of the given size, starting at the given offset in the provided buffer.
 */
extern int cbkMPEG2DBufRead (int *s32BufLen, unsigned char **pu8Buf,
                             int s32Offset, void *pvAppContext);

/*Added for call back by changes*/

void Mpeg2_register_func(sMpeg2DecObject *,int (*) (int *, unsigned char **, int , void *));

/*Added to call back by changes*/


#endif  /* ifndef DECODERINTERFACE_H */
