/*
 ***********************************************************************
 * Copyright 2005-2006 by Freescale Semiconductor, Inc.
 * All modifications are confidential and proprietary information
 * of Freescale Semiconductor, Inc. ALL RIGHTS RESERVED.
 ***********************************************************************
 */

#ifndef SMARTDSP_H
#define SMARTDSP_H

/*==================================================================================================
     Header Name: smartdsp.h
     General Description: Header file for SmartDSP API structures and types. 
====================================================================================================

====================================================================================================
                                         INCLUDE FILES
==================================================================================================*/

#ifndef __TYPEDEF_H

#ifdef _MW_SCC_VER_
#include "prototype.h"
#endif

#endif  /* __TYPEDEF_H */
/*==================================================================================================
                                           CONSTANTS
==================================================================================================*/
#define STD_RETURN_SHFREE   0
/* successful sub-processing operation and shared inter-phase memory released */

#define STD_RETURN_SHLOCK   1
/* successful sub-processing operation and shared inter-phase memory reserved */

#define STD_RETURN_SUCCESS  0
/* successful operation */

#define STD_RETURN_MORED    2
/* successful operation and more input required by current phase */

#define STD_RETURN_AGAIN    4
/* successful operation and more output from current phase */

#define STD_RETURN_OORPAR   16
/* parameter out of range */

#define STD_RETURN_ALNPAR   18
/* parameter not aligned */

#define STD_RETURN_ILLOPM   20
/* illegal operating mode (a fatal error occurred); instance must be closed */

#define STD_RETURN_NOTIMP   22
/* command not implemented */

#define STD_RETURN_ERRBASE (1<<8)

#define STD_RETURN_NRMBASE (1<<16)

#define STD_MEM_SCRATCH 0
/* scratch memory index */

#define STD_MEM_SHARED  1
/* shared memory index */

#define STD_MEM_MODREL  2
/* run-time relocatable data */

/*return types supported by the process function*/
#define STD_MODE_FRAME      0
/* return every frame */
#define STD_MODE_BUFF       1
/* return every buffer */
#define STD_MODE_NEED       2
/*return only when a buffer insufficiency error occurred*/

 

/*==================================================================================================
                                            MACROS
==================================================================================================*/

/*==================================================================================================
                                             ENUMS
==================================================================================================*/

/*==================================================================================================
                                 STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/
#ifndef __TYPEDEF_H

#ifndef __PROTOTYPE_H
typedef short int Word16;
typedef long int Word32;
typedef unsigned int UWord32;
#endif /* __PROTOTYPE_H */

typedef char Word8;
typedef unsigned char UWord8;
typedef unsigned short int UWord16;
typedef char Bool;

#endif  /* __TYPEDEF_H */


typedef UWord32 STD_RETURN_T;

typedef struct STD_MEMREF_S
{
	void *       pPtr;
    UWord32      uliLen;

} STD_MEMREF_T;

typedef struct STD_BD_S{
    void *       pPtr;
    UWord32      uliLen;
    UWord32      uliFill;
} STD_BD_T;


typedef struct STD_BD_SET_T
{
	STD_BD_T *   pPtr;
	UWord16      usiCntBDSet;
} STD_BD_SET_T;

typedef struct STD_VERSION_INFO_S
{
    UWord32      uliVersionNumber;
} STD_VERSION_INFO_T;

/*==================================================================================================
                                 GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/

/*==================================================================================================
                                     FUNCTION PROTOTYPES
==================================================================================================*/

/*================================================================================================*/

#endif /* SMARTDSP_H */

